@@ -1,8 +1,7 @@
 /* libmondo-archive-EXT.h */
 
 
-extern int archive_this_fileset(char *filelist,
-								char *fname, int setno);
+extern int archive_this_fileset(char *filelist, char *fname, int setno);
 extern int backup_data();
 extern int call_mindi_to_supply_boot_disks();
 extern bool can_we_fit_these_files_on_media(
@@ -34,7 +33,4 @@ extern int write_iso_and_go_on(bool last_cd);
 extern int write_final_iso_if_necessary();
 extern int call_growisofs(char *destfile);
 extern int make_afioballs_and_images_SINGLETHREAD();
-extern int archive_this_fileset_with_star(
-										  char *filelist, char *fname,
-										  int setno);
 extern void setenv_mondo_share(void);
