@@ -1,5 +1,5 @@
 /* libmondo-fork.c
-   $Id: libmondo-fork.c 3060 2012-11-10 04:05:37Z bruno $
+   $Id: libmondo-fork.c 3568 2016-04-16 14:00:51Z bruno $
 
 - subroutines for handling forking/pthreads/etc.
 */
@@ -7,6 +7,8 @@
 
 #include "my-stuff.h"
 #include "mr_mem.h"
+#include "mr_str.h"
+#include "mr_sys.h"
 #include "mondostructures.h"
 #include "libmondo-fork.h"
 #include "libmondo-string-EXT.h"
@@ -16,11 +18,12 @@
 #include "lib-common-externs.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-fork.c 3060 2012-11-10 04:05:37Z bruno $";
+//static char cvsid[] = "$Id: libmondo-fork.c 3568 2016-04-16 14:00:51Z bruno $";
 
 extern t_bkptype g_backup_media_type;
 extern bool g_text_mode;
 extern char *MONDO_LOGFILE;
+extern char *MONDO_LOGFILENAME;
 
 /* Reference to global bkpinfo */
 extern struct s_bkpinfo *bkpinfo;
@@ -37,41 +40,37 @@ char *call_program_and_get_last_line_of_output(char *call)
 {
 	/*@ buffers ***************************************************** */
 	static char result[MAX_STR_LEN];
-	char *tmp;
-	char *p;
+	char *tmp = NULL;
 
 	/*@ pointers **************************************************** */
-	FILE *fin;
+	FILE *fin = NULL;
 
 	/*@ initialize data ********************************************* */
-	malloc_string(tmp);
 	result[0] = '\0';
-	tmp[0] = '\0';
 
 	/*@******************************************************************** */
 
 	assert_string_is_neither_NULL_nor_zerolength(call);
+	log_msg(5, "cpaglloo called with '%s'", call);
 	if ((fin = popen(call, "r"))) {
-		for (p = fgets(tmp, MAX_STR_LEN, fin); !feof(fin) && (p != NULL);
-			 p = fgets(tmp, MAX_STR_LEN, fin)) {
+		while (!feof(fin)) {
+			mr_getline(tmp, fin);
 			if (strlen(tmp) > 1) {
 				strcpy(result, tmp);
+				log_msg(7, "result = '%s'", result);
 			}
+			mr_free(tmp);
 		}
 		paranoid_pclose(fin);
 	} else {
-		log_OS_error("Unable to popen call");
+		log_OS_error("Unable to open resulting file");
 	}
 	strip_spaces(result);
-	paranoid_free(tmp);
-	return (result);
+	log_msg(5, "cpaglloo returns '%s'", result);
+	return(result);
 }
 
 
-
-
-
-
 #define MONDO_POPMSG  "Your PC will not retract the CD tray automatically. Please call mondoarchive with the -m (manual CD tray) flag."
 
 /**
@@ -85,10 +84,8 @@ char *call_program_and_get_last_line_of_output(char *call)
  * - @c _ERR_ becomes the logfile (@p g_logfile)
  * @param isofile Replaces @c _ISO_ in @p basic_call. Should probably be the ISO image to create (-o parameter to mkisofs).
  * @param cd_no Replaces @c _CD#_ in @p basic_call. Should probably be the CD number.
- * @param logstub Unused.
  * @param what_i_am_doing The action taking place (e.g. "Making ISO #1"). Used as the title of the progress dialog.
  * @return Exit code of @c mkisofs (0 is success, anything else indicates failure).
- * @bug @p logstub is unused.
  */
 int
 eval_call_to_make_ISO(char *basic_call, char *isofile,
@@ -100,19 +97,22 @@ eval_call_to_make_ISO(char *basic_call, char *isofile,
 
 
 	/*@ buffers      *** */
-	char *midway_call, *ultimate_call, *tmp, *command, *incoming,
-		*old_stderr, *cd_number_str;
-	char *p;
+	char *midway_call = NULL;
+	char *ultimate_call = NULL;
+	char *tmp = NULL;
+		
+	char *cd_number_str = NULL;
+	char *command = NULL;
+	char *p= NULL;
 	char *tmp1 = NULL;
+	char *tmp2 = NULL;
 
 /*@***********   End Variables ***************************************/
 
 	log_msg(3, "Starting");
 	assert(bkpinfo != NULL);
-	// BERLIOS: doesn't work even if the string is correct !
-	//assert_string_is_neither_NULL_nor_zerolength(basic_call);
 	assert_string_is_neither_NULL_nor_zerolength(isofile);
-	assert_string_is_neither_NULL_nor_zerolength(logstub);
+
 	if (!(midway_call = malloc(1200))) {
 		fatal_error("Cannot malloc midway_call");
 	}
@@ -122,83 +122,66 @@ eval_call_to_make_ISO(char *basic_call, char *isofile,
 	if (!(tmp = malloc(1200))) {
 		fatal_error("Cannot malloc tmp");
 	}
-	if (!(command = malloc(1200))) {
-		fatal_error("Cannot malloc command");
-	}
-	malloc_string(incoming);
-	malloc_string(old_stderr);
-	malloc_string(cd_number_str);
-
-	incoming[0] = '\0';
-	old_stderr[0] = '\0';
 
 	if ((bkpinfo->netfs_user) && (strstr(bkpinfo->netfs_proto,"nfs"))) {
-		mr_asprintf(&tmp1, "su - %s -c \"%s\"", bkpinfo->netfs_user, basic_call);
+		mr_asprintf(tmp1, "su - %s -c \"%s\"", bkpinfo->netfs_user, basic_call);
 	} else {
-		mr_asprintf(&tmp1, "%s", basic_call);
+		mr_asprintf(tmp1, "%s", basic_call);
 	}
 
-	sprintf(cd_number_str, "%d", cd_no);
+	mr_asprintf(cd_number_str, "%d", cd_no);
 	resolve_naff_tokens(midway_call, tmp1, isofile, "_ISO_");
-	resolve_naff_tokens(tmp, midway_call, cd_number_str, "_CD#_");
-	resolve_naff_tokens(ultimate_call, tmp, MONDO_LOGFILE, "_ERR_");
 	log_msg(4, "basic call = '%s'", tmp1);
-	log_msg(4, "midway_call = '%s'", midway_call);
-	log_msg(4, "tmp = '%s'", tmp);
-	log_msg(4, "ultimate call = '%s'", ultimate_call);
-	sprintf(command, "%s >> %s", ultimate_call, MONDO_LOGFILE);
+	mr_free(tmp1);
 
-	log_to_screen
-		("Please be patient. Do not be alarmed by on-screen inactivity.");
-	log_msg(4, "Calling open_evalcall_form() with what_i_am_doing='%s'",
-			what_i_am_doing);
-	strcpy(tmp, command);
+	resolve_naff_tokens(tmp, midway_call, cd_number_str, "_CD#_");
+	log_msg(4, "midway_call = '%s'", midway_call);
+	mr_free(cd_number_str);
+
+	resolve_naff_tokens(ultimate_call, tmp, MONDO_LOGFILE, "_ERR_");
+	log_msg(4, "ultimate call = '%s'", ultimate_call);
+	mr_asprintf(command, "%s >> %s", ultimate_call, MONDO_LOGFILE);
+
+	/*  Copy the current logfile - of course incomplete to the media */
+	mr_system("gzip -c9 %s > ./archives/%d.%s", MONDO_LOGFILE, cd_number_str, MONDO_LOGFILENAME);
+
+	log_to_screen("Please be patient. Do not be alarmed by on-screen inactivity.");
+	log_msg(4, "Calling open_evalcall_form() with what_i_am_doing='%s'", what_i_am_doing);
 	if (bkpinfo->manual_cd_tray) {
-		p = strstr(tmp, "2>>");
+		mr_asprintf(tmp2, "%s", command);
+		p = strstr(tmp2, "2>>");
 		if (p) {
 			sprintf(p, "   ");
 			while (*p == ' ') {
 				p++;
 			}
-			for (; *p != ' '; p++) {
+			for (; (*p != ' ') && (*p != '\0'); p++) {
 				*p = ' ';
 			}
 		}
-		strcpy(command, tmp);
-#ifndef _XWIN
+		mr_free(command);
+		command = tmp2;
 		if (!g_text_mode) {
 			newtSuspend();
 		}
-#endif
 		log_msg(1, "command = '%s'", command);
 		retval += system(command);
 		if (!g_text_mode) {
 			newtResume();
 		}
 		if (retval) {
-			log_msg(2, "Basic call '%s' returned an error.", tmp1);
-			popup_and_OK("Press ENTER to continue.");
-			popup_and_OK
-				("mkisofs and/or cdrecord returned an error. CD was not created");
+			popup_and_OK("mkisofs and/or cdrecord returned an error. CD was not created");
 		}
 	}
 	/* if text mode then do the above & RETURN; if not text mode, do this... */
 	else {
 		log_msg(3, "command = '%s'", command);
-//      yes_this_is_a_goto:
-		retval =
-			run_external_binary_with_percentage_indicator_NEW
-			(what_i_am_doing, command);
+		retval = run_external_binary_with_percentage_indicator_NEW(what_i_am_doing, command);
 	}
-
-	mr_free(tmp1);
+	mr_free(command);
 	paranoid_free(midway_call);
 	paranoid_free(ultimate_call);
 	paranoid_free(tmp);
-	paranoid_free(command);
-	paranoid_free(incoming);
-	paranoid_free(old_stderr);
-	paranoid_free(cd_number_str);
 	return (retval);
 }
 
@@ -231,8 +214,7 @@ eval_call_to_make_USB(char *command, char *what_i_am_doing) {
 	}
 	log_msg(1, "command = '%s'", command);
 	if (!g_text_mode) {
-		retval = run_external_binary_with_percentage_indicator_NEW
-			(what_i_am_doing, command);
+		retval = run_external_binary_with_percentage_indicator_NEW(what_i_am_doing, command);
 	} else {
 		retval += system(command);
 	}
@@ -274,17 +256,14 @@ int run_program_and_log_output(char *program, int debug_level)
 	assert(program != NULL);
 	if (!program[0]) {
 		log_msg(2, "Warning - asked to run zerolength program");
-		return (1);
+		return(1);
 	}
-//  if (debug_level == TRUE) { debug_level=5; }
-
-	//  assert_string_is_neither_NULL_nor_zerolength(program);
 
 	if (debug_level <= g_loglevel) {
 		log_if_success = TRUE;
 		log_if_failure = TRUE;
 	}
-	mr_asprintf(&callstr, "%s > %s/mondo-run-prog-thing.tmp 2> %s/mondo-run-prog-thing.err", program, bkpinfo->tmpdir, bkpinfo->tmpdir);
+	mr_asprintf(callstr, "%s > %s/mondo-run-prog-thing.tmp 2> %s/mondo-run-prog-thing.err", program, bkpinfo->tmpdir, bkpinfo->tmpdir);
 	while ((p = strchr(callstr, '\r'))) {
 		*p = ' ';
 	}
@@ -300,17 +279,17 @@ int run_program_and_log_output(char *program, int debug_level)
 	}
 	mr_free(callstr);
 
-	mr_asprintf(&callstr, "cat %s/mondo-run-prog-thing.err >> %s/mondo-run-prog-thing.tmp 2> /dev/null", bkpinfo->tmpdir, bkpinfo->tmpdir);
+	mr_asprintf(callstr, "cat %s/mondo-run-prog-thing.err >> %s/mondo-run-prog-thing.tmp 2> /dev/null", bkpinfo->tmpdir, bkpinfo->tmpdir);
 	if (log_if_failure && system(callstr)) {
 		log_OS_error("Command failed");
 	}
 	mr_free(callstr);
 
-	mr_asprintf(&tmp1, "%s/mondo-run-prog-thing.err", bkpinfo->tmpdir);
+	mr_asprintf(tmp1, "%s/mondo-run-prog-thing.err", bkpinfo->tmpdir);
 	unlink(tmp1);
 	mr_free(tmp1);
 
-	mr_asprintf(&tmp1, "%s/mondo-run-prog-thing.tmp", bkpinfo->tmpdir);
+	mr_asprintf(tmp1, "%s/mondo-run-prog-thing.tmp", bkpinfo->tmpdir);
 	fin = fopen(tmp1, "r");
 	if (fin) {
 		for (q = fgets(incoming, MAX_STR_LEN, fin); !feof(fin) && (q != NULL); q = fgets(incoming, MAX_STR_LEN, fin)) {
@@ -321,7 +300,6 @@ int run_program_and_log_output(char *program, int debug_level)
 					p += 2;
 				}
 			}
-			/* end of patch */
 			strip_spaces(incoming);
 			if ((res == 0 && log_if_success) || (res != 0 && log_if_failure)) {
 				log_msg(0, incoming);
@@ -341,9 +319,7 @@ int run_program_and_log_output(char *program, int debug_level)
 			log_msg(0, "...ran just fine. :-)");
 		}
 	}
-//  else
-//    { log_msg (0, "-------------------------------ran w/ res=%d------------------------------", res); }
-	return (res);
+	return(res);
 }
 
 
@@ -367,32 +343,27 @@ int run_program_and_log_to_screen(char *basic_call, char *what_i_am_doing)
 	FILE *fin;
 
 	/*@ buffers **************************************************** */
-	char *tmp = NULL;
 	char *command = NULL;
-	char lockfile[MAX_STR_LEN];
+	char *lockfile = NULL;
 
 	/*@ end vars *************************************************** */
 
 	assert_string_is_neither_NULL_nor_zerolength(basic_call);
 
-	sprintf(lockfile, "%s/mojo-jojo.bla.bla", bkpinfo->tmpdir);
+	mr_asprintf(lockfile, "%s/mojo-jojo.bla.bla", bkpinfo->tmpdir);
 
-	mr_asprintf(&command,
-			"echo hi > %s ; %s >> %s 2>> %s; res=$?; sleep 1; rm -f %s; exit $res",
-			lockfile, basic_call, MONDO_LOGFILE, MONDO_LOGFILE, lockfile);
+	mr_asprintf(command, "echo hi > %s ; %s >> %s 2>> %s; res=$?; sleep 1; rm -f %s; exit $res", lockfile, basic_call, MONDO_LOGFILE, MONDO_LOGFILE, lockfile);
 	open_evalcall_form(what_i_am_doing);
-	mr_asprintf(&tmp, "Executing %s", basic_call);
-	log_msg(2, tmp);
-	mr_free(tmp);
+	log_msg(2, "Executing %s", basic_call);
 
 	if (!(fin = popen(command, "r"))) {
 		log_OS_error("Unable to popen-in command");
-		mr_asprintf(&tmp, "Failed utterly to call '%s'", command);
-		log_to_screen(tmp);
-		mr_free(tmp);
+		log_to_screen("Failed utterly to call '%s'", command);
 		mr_free(command);
+		mr_free(lockfile);
 		return (1);
 	}
+
 	if (!does_file_exist(lockfile)) {
 		log_to_screen("Waiting for '%s' to start",command);
 		for (i = 0; i < 60 && !does_file_exist(lockfile); sleep(1), i++) {
@@ -400,25 +371,12 @@ int run_program_and_log_to_screen(char *basic_call, char *what_i_am_doing)
 		}
 	}
 	mr_free(command);
-#ifdef _XWIN
-	/* This only can update when newline goes into the file,
-	   but it's *much* prettier/faster on Qt. */
-	while (does_file_exist(lockfile)) {
-		while (!feof(fin)) {
-			/* TODO: Dead and wrong code */
-			if (!fgets(tmp, 512, fin))
-				break;
-			log_to_screen(tmp);
-		}
-		usleep(500000);
-	}
-#else
-	/* This works on Newt, and it gives quicker updates. */
+
 	for (; does_file_exist(lockfile); sleep(1)) {
 		log_file_end_to_screen(MONDO_LOGFILE, "");
 		update_evalcall_form(1);
 	}
-#endif
+
 	/* Evaluate the status returned by pclose to get the exit code of the called program. */
 	errno = 0;
 	res = pclose(fin);
@@ -433,6 +391,8 @@ int run_program_and_log_to_screen(char *basic_call, char *what_i_am_doing)
 	}
 	close_evalcall_form();
 	unlink(lockfile);
+	mr_free(lockfile);
+
 	return (retval);
 }
 
@@ -452,9 +412,10 @@ int copy_from_src_to_dest(FILE * f_orig, FILE * f_archived, char direction)
 {
 // if dir=='w' then copy from orig to archived
 // if dir=='r' then copy from archived to orig
-	char *tmp;
-	char *buf;
-	char filestr[MAX_STR_LEN];
+	char *tmp = NULL;
+	char *tmp1 = NULL;
+	char *buf = NULL;
+	char *filestr = NULL;
 	long int bytes_to_be_read, bytes_read_in, bytes_written_out =
 		0, bufcap, subsliceno = 0;
 	int retval = 0;
@@ -464,10 +425,7 @@ int copy_from_src_to_dest(FILE * f_orig, FILE * f_archived, char direction)
 	int tmpcap = 512;
 
 	log_msg(5, "Opening.");
-	if (!(tmp = malloc(tmpcap))) {
-		fatal_error("Failed to malloc() tmp");
-	}
-	tmp[0] = '\0';
+
 	bufcap = 256L * 1024L;
 	if (!(buf = malloc(bufcap))) {
 		fatal_error("Failed to malloc() buf");
@@ -476,113 +434,142 @@ int copy_from_src_to_dest(FILE * f_orig, FILE * f_archived, char direction)
 	if (direction == 'w') {
 		fin = f_orig;
 		fout = f_archived;
-		sprintf(tmp, "%-64s", PIMP_START_SZ);
+		mr_asprintf(tmp, "%-64s", PIMP_START_SZ);
 		if (fwrite(tmp, 1, 64, fout) != 64) {
+			mr_free(tmp);
 			fatal_error("Can't write the introductory block");
 		}
+		mr_free(tmp);
+
 		while (1) {
 			bytes_to_be_read = bytes_read_in = fread(buf, 1, bufcap, fin);
 			if (bytes_read_in == 0) {
 				break;
 			}
-			sprintf(tmp, "%-64ld", bytes_read_in);
+			mr_asprintf(tmp, "%-64ld", bytes_read_in);
 			if (fwrite(tmp, 1, 64, fout) != 64) {
+				mr_free(tmp);
 				fatal_error("Cannot write introductory block");
 			}
+			mr_free(tmp);
+
 			log_msg(7,
 					"subslice #%ld --- I have read %ld of %ld bytes in from f_orig",
 					subsliceno, bytes_read_in, bytes_to_be_read);
 			bytes_written_out += fwrite(buf, 1, bytes_read_in, fout);
-			sprintf(tmp, "%-64ld", subsliceno);
+			mr_asprintf(tmp, "%-64ld", subsliceno);
 			if (fwrite(tmp, 1, 64, fout) != 64) {
+				mr_free(tmp);
 				fatal_error("Cannot write post-thingy block");
 			}
+			mr_free(tmp);
 			log_msg(7, "Subslice #%d written OK", subsliceno);
 			subsliceno++;
 		}
-		sprintf(tmp, "%-64ld", 0L);
+		mr_asprintf(tmp, "%-64ld", 0L);
 		if (fwrite(tmp, 1, 64L, fout) != 64L) {
+			mr_free(tmp);
 			fatal_error("Cannot write final introductory block");
 		}
+		mr_free(tmp);
+
+		mr_asprintf(tmp, "%-64s", PIMP_END_SZ);
+		if (fwrite(tmp, 1, 64, fout) != 64) {
+			mr_free(tmp);
+			fatal_error("Can't write the final block");
+		}
+		mr_free(tmp);
 	} else {
 		fin = f_archived;
 		fout = f_orig;
-		if (fread(tmp, 1, 64L, fin) != 64L) {
+		if (!(tmp1 = malloc(tmpcap))) {
+			fatal_error("Failed to malloc() tmp");
+		}
+		if (fread(tmp1, 1, 64L, fin) != 64L) {
+			mr_free(tmp1);
 			fatal_error("Cannot read the introductory block");
 		}
-		log_msg(5, "tmp is %s", tmp);
-		if (!strstr(tmp, PIMP_START_SZ)) {
+		log_msg(5, "tmp1 is %s", tmp1);
+		if (!strstr(tmp1, PIMP_START_SZ)) {
+			mr_free(tmp1);
 			fatal_error("Can't find intro blk");
 		}
-		if (fread(tmp, 1, 64L, fin) != 64L) {
+		if (fread(tmp1, 1, 64L, fin) != 64L) {
+			mr_free(tmp1);
 			fatal_error("Cannot read introductory blk");
 		}
-		bytes_to_be_read = atol(tmp);
+		bytes_to_be_read = atol(tmp1);
 		while (bytes_to_be_read > 0) {
-			log_msg(7, "subslice#%ld, bytes=%ld", subsliceno,
-					bytes_to_be_read);
+			log_msg(7, "subslice#%ld, bytes=%ld", subsliceno, bytes_to_be_read);
 			bytes_read_in = fread(buf, 1, bytes_to_be_read, fin);
 			if (bytes_read_in != bytes_to_be_read) {
-				fatal_error
-					("Danger, WIll Robinson. Failed to read whole subvol from archives.");
+				mr_free(tmp1);
+				fatal_error("Danger, WIll Robinson. Failed to read whole subvol from archives.");
 			}
 			bytes_written_out += fwrite(buf, 1, bytes_read_in, fout);
-			if (fread(tmp, 1, 64, fin) != 64) {
+			if (fread(tmp1, 1, 64, fin) != 64) {
+				mr_free(tmp1);
 				fatal_error("Cannot read post-thingy block");
 			}
-			if (atol(tmp) != subsliceno) {
-				log_msg(1, "Wanted subslice %ld but got %ld ('%s')",
-						subsliceno, atol(tmp), tmp);
+			if (atol(tmp1) != subsliceno) {
+				log_msg(1, "Wanted subslice %ld but got %ld ('%s')", subsliceno, atol(tmp1), tmp1);
 			}
 			log_msg(7, "Subslice #%ld read OK", subsliceno);
 			subsliceno++;
-			if (fread(tmp, 1, 64, fin) != 64) {
+			if (fread(tmp1, 1, 64, fin) != 64) {
+				mr_free(tmp1);
 				fatal_error("Cannot read introductory block");
 			}
-			bytes_to_be_read = atol(tmp);
+			bytes_to_be_read = atol(tmp1);
 		}
 	}
 
-//  log_msg(4, "Written %ld of %ld bytes", bytes_written_out, bytes_read_in);
-
 	if (direction == 'w') {
-		sprintf(tmp, "%-64s", PIMP_END_SZ);
+		mr_asprintf(tmp, "%-64s", PIMP_END_SZ);
 		if (fwrite(tmp, 1, 64, fout) != 64) {
+			mr_free(tmp);
 			fatal_error("Can't write the final block");
 		}
+		mr_free(tmp);
 	} else {
-		log_msg(1, "tmpA is %s", tmp);
-		if (!strstr(tmp, PIMP_END_SZ)) {
-			if (fread(tmp, 1, 64, fin) != 64) {
+		log_msg(1, "tmpA is %s", tmp1);
+		if (!strstr(tmp1, PIMP_END_SZ)) {
+			if (fread(tmp1, 1, 64, fin) != 64) {
+				mr_free(tmp1);
 				fatal_error("Can't read the final block");
 			}
-			log_msg(5, "tmpB is %s", tmp);
-			if (!strstr(tmp, PIMP_END_SZ)) {
-				sprintf(filestr, "%s/out.leftover", bkpinfo->tmpdir);
+			log_msg(5, "tmpB is %s", tmp1);
+			if (!strstr(tmp1, PIMP_END_SZ)) {
+				mr_asprintf(filestr, "%s/out.leftover", bkpinfo->tmpdir);
 				ftmp = fopen(filestr, "w");
-				bytes_read_in = fread(tmp, 1, 64, fin);
+				mr_free(filestr);
+
+				bytes_read_in = fread(tmp1, 1, 64, fin);
 				log_msg(1, "bytes_read_in = %ld", bytes_read_in);
-//      if (bytes_read_in!=128+64) { fatal_error("Can't read the terminating block"); }
+
 				if (fwrite(tmp, 1, bytes_read_in, ftmp)) {
 					fatal_error("Can't fwrite here");
 				}
-				sprintf(tmp, "I am here - %lld", (long long)ftello(fin));
-//    log_msg(0, tmp);
+
+				mr_asprintf(tmp, "I am here - %lld", (long long)ftello(fin));
 				if (fread(tmp, 1, tmpcap, fin)) {
+					mr_free(tmp);
 					fatal_error("Can't fread here");
 				}
 				log_msg(0, "tmp = '%s'", tmp);
 				if (fwrite(tmp, 1, tmpcap, ftmp)) {
+					mr_free(tmp);
 					fatal_error("Can't fwrite there");
 				}
 				fclose(ftmp);
+				mr_free(tmp);
 				fatal_error("Missing terminating block");
 			}
 		}
 	}
+	mr_free(tmp1);
 
 	paranoid_free(buf);
-	paranoid_free(tmp);
 	log_msg(3, "Successfully copied %ld bytes", bytes_written_out);
 	return (retval);
 }
@@ -600,7 +587,7 @@ int feed_into_ntfsprog(char *input_device, char *output_fname)
 {
 // BACKUP
 	int res = -1;
-	char*command;
+	char *command = NULL;
 
 	if (!does_file_exist(input_device)) {
 		fatal_error("input device does not exist");
@@ -608,10 +595,10 @@ int feed_into_ntfsprog(char *input_device, char *output_fname)
 	if ( !find_home_of_exe("ntfsclone")) {
 		fatal_error("ntfsclone not found");
 	}
-	malloc_string(command);
-	sprintf(command, "ntfsclone --rescue --force --save-image --overwrite %s %s", output_fname, input_device);
+	mr_asprintf(command, "ntfsclone --rescue --force --save-image --overwrite %s %s", output_fname, input_device);
 	res = run_program_and_log_output(command, 5);
-	paranoid_free(command);
+	mr_free(command);
+
 	unlink(output_fname);
 	return (res);
 }
@@ -637,20 +624,17 @@ void *run_prog_in_bkgd_then_exit(void *info)
 
 
 
-int run_external_binary_with_percentage_indicator_NEW(char *tt, char *cmd)
-{
+int run_external_binary_with_percentage_indicator_NEW(char *tt, char *cmd) {
 
 	/*@ int *************************************************************** */
 	int res = 0;
 	int percentage = 0;
 	int maxpc = 100;
 	int pcno = 0;
-	int last_pcno = 0;
 	int counter = 0;
 
 	/*@ buffers *********************************************************** */
-	char *command;
-	char *title;
+	char *command = NULL;
 	/*@ pointers ********************************************************** */
 	static int chldres = 0;
 	int *pchild_result;
@@ -661,38 +645,34 @@ int run_external_binary_with_percentage_indicator_NEW(char *tt, char *cmd)
 	assert_string_is_neither_NULL_nor_zerolength(tt);
 	*pchild_result = 999;
 
-	malloc_string(title);
-	malloc_string(command);
-	strcpy(title, tt);
-	sprintf(command, "%s 2>> %s", cmd, MONDO_LOGFILE);
+	mr_asprintf(command, "%s 2>> %s", cmd, MONDO_LOGFILE);
 	log_msg(3, "command = '%s'", command);
-	if ((res =
-		 pthread_create(&childthread, NULL, run_prog_in_bkgd_then_exit,
-						(void *) command))) {
+	if ((res = pthread_create(&childthread, NULL, run_prog_in_bkgd_then_exit, (void *) command))) {
 		fatal_error("Unable to create an archival thread");
 	}
 
 	log_msg(8, "Parent running");
-	open_evalcall_form(title);
+	open_evalcall_form(tt);
+
 	for (sleep(1); command[0] != '\0'; sleep(1)) {
-		pcno = grab_percentage_from_last_line_of_file(MONDO_LOGFILE);
-		if (pcno <= 0 || pcno > 100) {
-			log_msg(8, "Weird pc#");
+		if (strstr(cmd,"mindi") != NULL) {
+			pcno = grab_percentage_from_last_line_of_file(MINDI_RUNFILE);
+		} else {
+			pcno = grab_percentage_from_last_line_of_file(MONDO_LOGFILE);
+		}
+		if (pcno < 0 || pcno > 100) {
+			log_msg(8, "Weird pc# %d", pcno);
 			continue;
 		}
 		percentage = pcno * 100 / maxpc;
-		if (pcno <= 5 && last_pcno >= 40) {
-			close_evalcall_form();
-			strcpy(title, "Verifying...");
-			open_evalcall_form(title);
-		}
 		if (counter++ >= 5) {
 			counter = 0;
 			log_file_end_to_screen(MONDO_LOGFILE, "");
 		}
-		last_pcno = pcno;
 		update_evalcall_form(percentage);
 	}
+	mr_free(command);
+
 	log_file_end_to_screen(MONDO_LOGFILE, "");
 	close_evalcall_form();
 	pthread_join(childthread, (void *) (&pchild_result));
@@ -702,14 +682,10 @@ int run_external_binary_with_percentage_indicator_NEW(char *tt, char *cmd)
 		res = 666;
 	}
 	log_msg(3, "Parent res = %d", res);
-	paranoid_free(command);
-	paranoid_free(title);
 	return (res);
 }
 
 
-
-
 /**
  * Feed @p input_fifo through ntfsclone (restore) to @p output_device.
  * @param input_fifo The ntfsclone file to read.
@@ -720,14 +696,13 @@ int feed_outfrom_ntfsprog(char *output_device, char *input_fifo)
 {
 // RESTORE
 	int res = -1;
-	char *command;
+	char *command = NULL;
 
 	if ( !find_home_of_exe("ntfsclone")) {
 		fatal_error("ntfsclone not found");
 	}
-	malloc_string(command);
-	sprintf(command, "ntfsclone --rescue --force --restore-image --overwrite %s %s", output_device, input_fifo);
+	mr_asprintf(command, "ntfsclone --rescue --force --restore-image --overwrite %s %s", output_device, input_fifo);
 	res = run_program_and_log_output(command, 5);
-	paranoid_free(command);
+	mr_free(command);
 	return (res);
 }
