@@ -1,36 +1,5 @@
 /* libmondo-raid.c                                                subroutines for handling RAID
-   $Id: libmondo-raid.c 1261 2007-03-21 23:29:27Z bruno $
-.
-
-
-06/29
-- added create_raidtab_from_mdstat()
-- changed char[MAX_STR_LEN] to char*
-
-10/21/2003
-- get_next_raidtab_line() --- correctly handle multiple spaces
-  between label and value
-
-07/03
-- line 447 - changed assert()
-
-05/08
-- cleaned up some FreeBSd-specific stuff
-
-05/05
-- added Joshua Oreman's FreeBSD patches
-
-04/25
-- added a bunch of RAID utilities from mondorestore/mondo-restore.c
-
-04/24/2003
-- added some assert()'s and log_OS_error()'s
-
-10/19/2002
-- added some comments
-
-07/24
-- created
+   $Id: libmondo-raid.c 3566 2016-04-13 10:58:26Z bruno $
 */
 
 
@@ -45,9 +14,11 @@
 #include "libmondo-files-EXT.h"
 #include "libmondo-tools-EXT.h"
 #include "libmondo-string-EXT.h"
+#include "libmondo-fork-EXT.h"
 #include "lib-common-externs.h"
 #include "libmondo-raid.h"
-#include "mr_string.h"
+#include "mr_mem.h"
+#include "mr_str.h"
 
 #ifdef __FreeBSD__
 /* Nonstandard library functions: */
@@ -56,7 +27,7 @@ extern char *strsep(char **stringp, const char *delim);
 #endif
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-raid.c 1261 2007-03-21 23:29:27Z bruno $";
+//static char cvsid[] = "$Id: libmondo-raid.c 3566 2016-04-13 10:58:26Z bruno $";
 
 
 /**
@@ -76,20 +47,20 @@ bool is_this_raid_personality_registered(int raidno)
 			|| (raidno == 5)) ? TRUE : FALSE;
 #else
 	/*@ buffer ********************************************************** */
-	char *command;
+	char *command = NULL;
 	int res;
 
-	command = malloc(MAX_STR_LEN * 2);
-	strcpy(command, "grep \"");
+	mr_asprintf(command, "grep \"");
 	if (raidno == -1) {
-		strcat(command, "linear");
+		mr_strcat(command, "linear");
 	} else {
-		sprintf(command + strlen(command), "raid%d", raidno);
+		mr_strcat(command, "raid%d", raidno);
 	}
-	strcat(command, "\" /proc/mdstat > /dev/null 2> /dev/null");
+	mr_strcat(command, "\" /proc/mdstat > /dev/null 2> /dev/null");
 	log_it("Is raid %d registered? Command = '%s'", raidno, command);
 	res = system(command);
 	paranoid_free(command);
+
 	if (res) {
 		return (FALSE);
 	} else {
@@ -221,16 +192,15 @@ write_variableINT_to_raid_var_line(struct raid_device_record *raidrec,
 								   int lino, char *label, int value)
 {
 	/*@ buffers ***************************************************** */
-	char *sz_value;
+	char *sz_value = NULL;
 
-	malloc_string(sz_value);
 	assert(raidrec != NULL);
 	assert(label != NULL);
 
-	sprintf(sz_value, "%d", value);
+	mr_asprintf(sz_value, "%d", value);
 	strcpy(raidrec->additional_vars.el[lino].label, label);
 	strcpy(raidrec->additional_vars.el[lino].value, sz_value);
-	paranoid_free(sz_value);
+	mr_free(sz_value);
 }
 #endif
 
@@ -272,6 +242,7 @@ void add_disk_to_raid_device(struct list_of_disks *disklist,
 	disklist->el[items].index = index;
 	items++;
 	disklist->entries = items;
+	log_msg(3, "Adding device %s (%d)", device_to_add, index);
 }
 #endif
 
@@ -364,30 +335,31 @@ void save_raidrec_to_file(struct
 {
 #ifdef __FreeBSD__
 	int i, j;
+	char *org = NULL;
 
 	fprintf(fout, "\nvolume %s\n", raidrec->volname);
 	for (i = 0; i < raidrec->plexes; ++i) {
-		char org[24];
 		switch (raidrec->plex[i].raidlevel) {
 		case -1:
-			strcpy(org, "concat");
+			mr_asprintf(org, "concat");
 			break;
 		case 0:
-			strcpy(org, "striped");
+			mr_asprintf(org, "striped");
 			break;
 		case 5:
-			strcpy(org, "raid5");
+			mr_asprintf(org, "raid5");
 			break;
 		}
 		fprintf(fout, "  plex org %s", org);
+		mr_free(org);
+
 		if (raidrec->plex[i].raidlevel != -1) {
 			fprintf(fout, " %ik", raidrec->plex[i].stripesize);
 		}
 		fprintf(fout, "\n");
 
 		for (j = 0; j < raidrec->plex[i].subdisks; ++j) {
-			fprintf(fout, "    sd drive %s size 0\n",
-					raidrec->plex[i].sd[j].which_device);
+			fprintf(fout, "    sd drive %s size 0\n", raidrec->plex[i].sd[j].which_device);
 		}
 	}
 #else
@@ -400,21 +372,16 @@ void save_raidrec_to_file(struct
 	} else if (raidrec->raid_level == -1) {
 		fprintf(fout, "    raid-level            linear\n");
 	} else {
-		fprintf(fout, "    raid-level            %d\n",
-				raidrec->raid_level);
+		fprintf(fout, "    raid-level            %d\n", raidrec->raid_level);
 	}
-	fprintf(fout, "    nr-raid-disks         %d\n",
-			raidrec->data_disks.entries);
+	fprintf(fout, "    nr-raid-disks         %d\n", raidrec->data_disks.entries);
 	if (raidrec->spare_disks.entries > 0) {
-		fprintf(fout, "    nr-spare-disks        %d\n",
-				raidrec->spare_disks.entries);
+		fprintf(fout, "    nr-spare-disks        %d\n", raidrec->spare_disks.entries);
 	}
 	if (raidrec->parity_disks.entries > 0) {
-		fprintf(fout, "    nr-parity-disks       %d\n",
-				raidrec->parity_disks.entries);
+		fprintf(fout, "    nr-parity-disks       %d\n", raidrec->parity_disks.entries);
 	}
-	fprintf(fout, "    persistent-superblock %d\n",
-			raidrec->persistent_superblock);
+	fprintf(fout, "    persistent-superblock %d\n", raidrec->persistent_superblock);
 	if (raidrec->chunk_size > -1) {
 	  fprintf(fout, "    chunk-size            %d\n", raidrec->chunk_size);
 	}
@@ -454,35 +421,32 @@ void save_raidrec_to_file(struct
  * @param value Where to put the line's value.
  * @return 0 if the line was read and stored successfully, 1 if we're at end of file.
  */
-int get_next_raidtab_line(FILE * fin, char *label, char *value)
+static int get_next_raidtab_line(FILE * fin, char **label, char **value)
 {
-	char *incoming;
-	char *p;
+	char *incoming = NULL;
+	char *p = NULL;
 
-	malloc_string(incoming);
 	assert(fin != NULL);
-	assert(label != NULL);
-	assert(value != NULL);
 
-	label[0] = value[0] = '\0';
 	if (feof(fin)) {
-		paranoid_free(incoming);
 		return (1);
 	}
-	for (fgets(incoming, MAX_STR_LEN - 1, fin); !feof(fin);
-		 fgets(incoming, MAX_STR_LEN - 1, fin)) {
-		strip_spaces(incoming);
+
+	for (mr_getline(incoming, fin); !feof(fin); mr_getline(incoming, fin)) {
+		log_msg(10,"Found in raidtab line: %s",incoming);
+		mr_strip_spaces(incoming);
 		p = strchr(incoming, ' ');
 		if (strlen(incoming) < 3 || incoming[0] == '#' || !p) {
+			mr_free(incoming);
 			continue;
 		}
 		*(p++) = '\0';
 		while (*p == ' ') {
 			p++;
 		}
-		strcpy(label, incoming);
-		strcpy(value, p);
-		paranoid_free(incoming);
+		*label = incoming;
+		*value = p;
+		log_msg(10,"Found in raidtab pair: %s|%s",*label,*value);
 		return (0);
 	}
 	return (1);
@@ -497,25 +461,20 @@ int get_next_raidtab_line(FILE * fin, char *label, char *value)
  * @return 0 for success, 1 for failure.
  */
 #ifdef __FreeBSD__
-int load_raidtab_into_raidlist(struct raidlist_itself *raidlist,
-							   char *fname)
+int load_raidtab_into_raidlist(struct raidlist_itself *raidlist, char *fname)
 {
 	FILE *fin;
-	char *tmp;
 	int items;
 
-	malloc_string(tmp);
 	raidlist->spares.entries = 0;
 	raidlist->disks.entries = 0;
 	if (length_of_file(fname) < 5) {
 		log_it("Raidtab is very small or non-existent. Ignoring it.");
 		raidlist->entries = 0;
-		paranoid_free(tmp);
 		return (0);
 	}
 	if (!(fin = fopen(fname, "r"))) {
 		log_it("Cannot open raidtab");
-		paranoid_free(tmp);
 		return (1);
 	}
 	items = 0;
@@ -535,20 +494,14 @@ int load_raidtab_into_raidlist(struct raidlist_itself *raidlist,
 				continue;
 
 			if (get_option_state(argc, argv, "hotspare")) {
-				strcpy(raidlist->spares.el[raidlist->spares.entries].name,
-					   drivename);
-				strcpy(raidlist->spares.el[raidlist->spares.entries].
-					   device, devname);
-				raidlist->spares.el[raidlist->spares.entries].index =
-					raidlist->disks.entries;
+				strcpy(raidlist->spares.el[raidlist->spares.entries].name, drivename);
+				strcpy(raidlist->spares.el[raidlist->spares.entries].device, devname);
+				raidlist->spares.el[raidlist->spares.entries].index = raidlist->disks.entries;
 				raidlist->spares.entries++;
 			} else {
-				strcpy(raidlist->disks.el[raidlist->disks.entries].name,
-					   drivename);
-				strcpy(raidlist->disks.el[raidlist->disks.entries].device,
-					   devname);
-				raidlist->disks.el[raidlist->disks.entries].index =
-					raidlist->disks.entries;
+				strcpy(raidlist->disks.el[raidlist->disks.entries].name, drivename);
+				strcpy(raidlist->disks.el[raidlist->disks.entries].device, devname);
+				raidlist->disks.el[raidlist->disks.entries].index = raidlist->disks.entries;
 				raidlist->disks.entries++;
 			}
 		} else if (!strcmp(argv[0], "volume")) {
@@ -618,49 +571,35 @@ int load_raidtab_into_raidlist(struct raidlist_itself *raidlist,
 	}
 	fclose(fin);
 	log_it("Raidtab loaded successfully.");
-	sprintf(tmp, "%d RAID devices in raidtab", raidlist->entries);
-	log_it(tmp);
-	paranoid_free(tmp);
+	log_it("%d RAID devices in raidtab", raidlist->entries);
 	return (0);
 }
 
 
 #else
 
-int load_raidtab_into_raidlist(struct raidlist_itself *raidlist,
-							   char *fname)
-{
-	FILE *fin;
-	char *tmp;
-	char *label;
-	char *value;
-	int items;
-	int v;
+int load_raidtab_into_raidlist(struct raidlist_itself *raidlist, char *fname) {
+
+	FILE *fin = NULL;
+	char *label = NULL;
+	char *value = NULL;
+	int items = 0;
+	int v = 0;
 
-	malloc_string(tmp);
-	malloc_string(label);
-	malloc_string(value);
 	assert(raidlist != NULL);
 	assert_string_is_neither_NULL_nor_zerolength(fname);
 
 	if (length_of_file(fname) < 5) {
 		log_it("Raidtab is very small or non-existent. Ignoring it.");
 		raidlist->entries = 0;
-		paranoid_free(tmp);
-		paranoid_free(label);
-		paranoid_free(value);
 		return (0);
 	}
 	if (!(fin = fopen(fname, "r"))) {
 		log_it("Cannot open raidtab");
-		paranoid_free(tmp);
-		paranoid_free(label);
-		paranoid_free(value);
 		return (1);
 	}
-	items = 0;
 	log_it("Loading raidtab...");
-	get_next_raidtab_line(fin, label, value);
+	get_next_raidtab_line(fin, &label, &value);
 	while (!feof(fin)) {
 		log_msg(1, "Looking for raid record #%d", items);
 		initialize_raidrec(&raidlist->el[items]);
@@ -669,10 +608,12 @@ int load_raidtab_into_raidlist(struct raidlist_itself *raidlist,
 		while (!feof(fin) && strcmp(label, "raiddev")) {
 			strcpy(raidlist->el[items].additional_vars.el[v].label, label);
 			strcpy(raidlist->el[items].additional_vars.el[v].value, value);
+			mr_free(label);
+			log_msg(2,"Found raidtab entry Label: %s Value: %s",raidlist->el[items].additional_vars.el[v].label,raidlist->el[items].additional_vars.el[v].value);
 			v++;
-			get_next_raidtab_line(fin, label, value);
-			log_it(tmp);
+			get_next_raidtab_line(fin, &label, &value);
 		}
+
 		raidlist->el[items].additional_vars.entries = v;
 		if (feof(fin)) {
 			log_msg(1, "No more records.");
@@ -680,20 +621,18 @@ int load_raidtab_into_raidlist(struct raidlist_itself *raidlist,
 		}
 		log_msg(2, "Record #%d (%s) found", items, value);
 		strcpy(raidlist->el[items].raid_device, value);
-		for (get_next_raidtab_line(fin, label, value);
-			 !feof(fin) && strcmp(label, "raiddev");
-			 get_next_raidtab_line(fin, label, value)) {
+		mr_free(label);
+		for (get_next_raidtab_line(fin, &label, &value); !feof(fin) && strcmp(label, "raiddev"); get_next_raidtab_line(fin, &label, &value)) {
 			process_raidtab_line(fin, &raidlist->el[items], label, value);
+			mr_free(label);
 		}
 		items++;
 	}
+	mr_free(label);
 	paranoid_fclose(fin);
 	raidlist->entries = items;
 	log_msg(1, "Raidtab loaded successfully.");
 	log_msg(1, "%d RAID devices in raidtab", items);
-	paranoid_free(tmp);
-	paranoid_free(label);
-	paranoid_free(value);
 	return (0);
 }
 #endif
@@ -714,23 +653,17 @@ int load_raidtab_into_raidlist(struct raidlist_itself *raidlist,
  * @param value Where to put the value processed.
  */
 void
-process_raidtab_line(FILE * fin,
-					 struct raid_device_record *raidrec,
-					 char *label, char *value)
+process_raidtab_line(FILE * fin, struct raid_device_record *raidrec, char *label, char *value)
 {
 
 	/*@ add mallocs * */
-	char *tmp;
-	char *labelB;
-	char *valueB;
+	char *labelB = NULL;
+	char *valueB = NULL;
 
 	struct list_of_disks *disklist;
 	int index;
 	int v;
 
-	malloc_string(tmp);
-	malloc_string(labelB);
-	malloc_string(valueB);
 	assert(fin != NULL);
 	assert(raidrec != NULL);
 	assert_string_is_neither_NULL_nor_zerolength(label);
@@ -744,6 +677,7 @@ process_raidtab_line(FILE * fin,
 		} else {
 			raidrec->raid_level = atoi(value);
 		}
+		log_msg(4,"Found raid level %d",raidrec->raid_level);
 	} else if (!strcmp(label, "nr-raid-disks")) {	/* ignore it */
 	} else if (!strcmp(label, "nr-spare-disks")) {	/* ignore it */
 	} else if (!strcmp(label, "nr-parity-disks")) {	/* ignore it */
@@ -762,10 +696,11 @@ process_raidtab_line(FILE * fin,
 		} else if (!strcmp(value, "right-symmetric")) {
 			raidrec->parity = 3;
 		} else {
-			log_msg(1, "Unknown RAID parity algorithm '%s'\n.", value);
+			log_msg(1, "Unknown RAID parity algorithm '%s'", value);
 		}
+		log_msg(4,"Found raid parity %d",raidrec->parity);
 	} else if (!strcmp(label, "device")) {
-		get_next_raidtab_line(fin, labelB, valueB);
+		get_next_raidtab_line(fin, &labelB, &valueB);
 		if (!strcmp(labelB, "raid-disk")) {
 			disklist = &raidrec->data_disks;
 		} else if (!strcmp(labelB, "spare-disk")) {
@@ -778,23 +713,20 @@ process_raidtab_line(FILE * fin,
 			disklist = NULL;
 		}
 		if (!disklist) {
-			sprintf(tmp,
-					"Ignoring '%s %s' pair of disk %s", labelB, valueB,
-					label);
-			log_it(tmp);
+			log_it("Ignoring '%s %s' pair of disk %s", labelB, valueB, label);
 		} else {
 			index = atoi(valueB);
 			add_disk_to_raid_device(disklist, value, index);
 		}
+		mr_free(labelB);
 	} else {
 		v = raidrec->additional_vars.entries;
 		strcpy(raidrec->additional_vars.el[v].label, label);
 		strcpy(raidrec->additional_vars.el[v].value, value);
-		raidrec->additional_vars.entries = ++v;
+		log_msg(4,"Found additional raid pair #%d: %s / %s",v,raidrec->additional_vars.el[v].label,raidrec->additional_vars.el[v].value);
+		v++;
+		raidrec->additional_vars.entries = v;
 	}
-	paranoid_free(tmp);
-	paranoid_free(labelB);
-	paranoid_free(valueB);
 }
 #endif
 
@@ -806,8 +738,7 @@ process_raidtab_line(FILE * fin,
  * @param fout The stream to write to.
  */
 void
-save_disklist_to_file(char *listname,
-					  struct list_of_disks *disklist, FILE * fout)
+save_disklist_to_file(char *listname, struct list_of_disks *disklist, FILE * fout)
 {
 	int i;
 
@@ -816,8 +747,7 @@ save_disklist_to_file(char *listname,
 	assert(fout != NULL);
 
 	for (i = 0; i < disklist->entries; i++) {
-		fprintf(fout, "    device                %s\n",
-				disklist->el[i].device);
+		fprintf(fout, "    device                %s\n", disklist->el[i].device);
 		fprintf(fout, "    %-21s %d\n", listname, disklist->el[i].index);
 	}
 }
@@ -850,12 +780,9 @@ char **get_next_vinum_conf_line(FILE * f, int *argc)
 	int cnt = 0;
 	static char *argv[64];
 	char **ap;
-	char *line = (char *) malloc(MAX_STR_LEN);
-	if (!line)
-		errx(1,
-			 "unable to allocate %i bytes of memory for `char *line' at %s:%i",
-			 MAX_STR_LEN, __FILE__, __LINE__);
-	(void) fgets(line, MAX_STR_LEN, f);
+	char *line = NULL;
+
+	mr_getline(line, f);
 	if (feof(f)) {
 		log_it("[GNVCL] Uh... I reached the EOF.");
 		return 0;
@@ -867,6 +794,7 @@ char **get_next_vinum_conf_line(FILE * f, int *argc)
 				break;
 			cnt++;
 		}
+	mr_free(line);
 
 	if (strchr(argv[cnt - 1], '\n')) {
 		*(strchr(argv[cnt - 1], '\n')) = '\0';
@@ -1010,253 +938,296 @@ long long size_spec(char *spec)
 
 
 
-int parse_mdstat(struct raidlist_itself *raidlist, char *device_prefix) {
+int parse_mdstat(char *mdstat_fname, struct raidlist_itself *raidlist, char *device_prefix) {
 
-  const char delims[] = " ";
+const char delims[] = " ";
 
-  FILE   *fin;
-  int    res = 0, row, i, index_min;
-  int lastpos = 0;
-  size_t len = 0;
-  char   *token;
-  char *string = NULL;
-  char *pos;
-  char type;
-  char *strtmp;
+FILE *fin = NULL;
+int res = 0, row, i, index_min;
+int v = 0;
+int lastpos = 0;
+size_t len = 0;
+char *token = NULL;
+char *string = NULL;
+char *cmd = NULL;
+char *pos = NULL;
+char type;
+char *strtmp = NULL;
+char *strtmp2 = NULL;
 
-  // open file
-  if (!(fin = fopen(MDSTAT_FILE, "r"))) {
-    log_msg(1, "Could not open %s.\n", MDSTAT_FILE);
-    return 1;
-  }
-  // initialise record, build progress and row counters
-  raidlist->entries = 0;
-  raidlist->el[raidlist->entries].progress = 999;
-  row = 1;
-  // skip first output row - contains registered RAID levels
-  res = getline(&string, &len, fin);
-  // parse the rest
-  while ( !feof_unlocked(fin) ) {
-    res = getline(&string, &len, fin);
-    if (res <= 0) break;
-    // trim leading spaces
-    pos = string;
-    while (*pos == ' ') pos += 1;
-    asprintf(&strtmp, pos);
-    strcpy(string, strtmp);
-    paranoid_free(strtmp);
-    // if we have newline after only spaces, this is a blank line, update
-    // counters, otherwise do normal parsing
-    if (*string == '\n') {
-      row = 1;
-      raidlist->entries++;
-      raidlist->el[raidlist->entries].progress = 999;
-    } else {
-      switch (row) {
-      case 1:  // device information
-	// check whether last line of record and if so skip
-	pos = strcasestr(string, "unused devices: ");
-	if (pos == string) {
-	  //raidlist->entries--;
-	  break;
-	}
-	// tokenise string
-	token = mr_strtok (string, delims, &lastpos);
-	// get RAID device name
-	asprintf(&strtmp,"%s%s", device_prefix, token);
-	strcpy(raidlist->el[raidlist->entries].raid_device, strtmp);
-	paranoid_free(strtmp);
-	paranoid_free(token);
-	// skip ':' and status
-	token = mr_strtok (string, delims, &lastpos);
-	paranoid_free(token);
-	token = mr_strtok (string, delims, &lastpos);
-	if (!strcmp(token, "inactive")) {
-	  log_msg(1, "RAID device '%s' inactive.\n",
-		 raidlist->el[raidlist->entries].raid_device);
-	  paranoid_free(string);
-	  paranoid_free(token);
-	  return 1;
-	}
-	paranoid_free(token);
-
-	// get RAID level
-	token = mr_strtok (string, delims, &lastpos);
-	if (!strcmp(token, "multipath")) {
-	  raidlist->el[raidlist->entries].raid_level = -2;
-	} else if (!strcmp(token, "linear")) {
-	  raidlist->el[raidlist->entries].raid_level = -1;
-	} else if (!strcmp(token, "raid0")) {
-	  raidlist->el[raidlist->entries].raid_level = 0;
-	} else if (!strcmp(token, "raid1")) {
-	  raidlist->el[raidlist->entries].raid_level = 1;
-	} else if (!strcmp(token, "raid4")) {
-	  raidlist->el[raidlist->entries].raid_level = 4;
-	} else if (!strcmp(token, "raid5")) {
-	  raidlist->el[raidlist->entries].raid_level = 5;
-	} else if (!strcmp(token, "raid6")) {
-	  raidlist->el[raidlist->entries].raid_level = 6;
-	} else if (!strcmp(token, "raid10")) {
-	  raidlist->el[raidlist->entries].raid_level = 10;
+// open file
+if (!(fin = fopen(mdstat_fname, "r"))) {
+	log_msg(1, "Could not open %s.", mdstat_fname);
+	return 1;
+}
+// initialise record, build progress and row counters
+raidlist->entries = 0;
+raidlist->el[raidlist->entries].progress = 999;
+row = 1;
+// skip first output row - contains registered RAID levels
+res = getline(&strtmp2, &len, fin);
+mr_free(strtmp2);
+// parse the rest
+while ( !feof_unlocked(fin) ) {
+	res = getline(&string, &len, fin);
+	log_msg(8, "mdstat line '%s' read.", string);
+	if (res <= 0) break;
+	// trim spaces
+	strip_spaces(string);
+	log_msg(8, "mdstat line 2 '%s' read.", string);
+	// if we have newline after only spaces, this is a blank line, update
+	// counters, otherwise do normal parsing
+	if (!strcmp(string,"")) {
+		row = 1;
+		raidlist->entries++;
+		raidlist->el[raidlist->entries].progress = 999;
 	} else {
-	  log_msg(1, "Unknown RAID level '%s'.\n", token);
-	  paranoid_free(string);
-	  paranoid_free(token);
-	  return 1;
-	}
-	paranoid_free(token);
+		switch (row) {
+		case 1:  // device information
+			// check whether last line of record and if so skip
+			log_msg(8, "This is the device line");
+			pos = strcasestr(string, "unused devices: ");
+			if (pos != NULL) {
+				break;
+			}
+			// tokenise string
+			token = mr_strtok(string, delims, &lastpos);
+			if (token == NULL) {
+				// should not happen !
+				break;
+			}
+			// get RAID device name
+			mr_asprintf(strtmp,"%s%s", device_prefix, token);
+			strncpy(raidlist->el[raidlist->entries].raid_device, strtmp, 63);
+			raidlist->el[raidlist->entries].raid_device[63] = '\0';
+			mr_free(strtmp);
+			mr_free(token);
+			// store the UUID value in the additional_vars structure
+			v = raidlist->el[raidlist->entries].additional_vars.entries;
+			strcpy(raidlist->el[raidlist->entries].additional_vars.el[v].label, "UUID");
+			mr_asprintf(cmd,"mdadm --detail %s | grep UUID | cut -d: -f2- | sed 's/^ *//'", raidlist->el[raidlist->entries].raid_device);
+			mr_asprintf(strtmp, "%s", call_program_and_get_last_line_of_output(cmd));
+			strcpy(raidlist->el[raidlist->entries].additional_vars.el[v].value, strtmp);
+			mr_free(strtmp);
+			v++;
+			// store the Version value in the additional_vars structure
+			strcpy(raidlist->el[raidlist->entries].additional_vars.el[v].label, "Version");
+			mr_asprintf(cmd,"mdadm --detail %s | grep Version | cut -d: -f2- | sed 's/^ *//'", raidlist->el[raidlist->entries].raid_device);
+			mr_asprintf(strtmp, "%s", call_program_and_get_last_line_of_output(cmd));
+			strcpy(raidlist->el[raidlist->entries].additional_vars.el[v].value, strtmp);
+			mr_free(strtmp);
+			v++;
+			raidlist->el[raidlist->entries].additional_vars.entries = v;
+			// skip ':' and status
+			token = mr_strtok(string, delims, &lastpos);
+			if (token == NULL) {
+				// should not happen !
+				break;
+			}
+			mr_free(token);
+			token = mr_strtok(string, delims, &lastpos);
+			if (token == NULL) {
+				// should not happen !
+				break;
+			}
+			if (!strcmp(token, "inactive")) {
+				log_msg(1, "RAID device '%s' inactive.", raidlist->el[raidlist->entries].raid_device);
+				mr_free(string);
+				mr_free(token);
+				return 1;
+			}
+			mr_free(token);
 
-	// get RAID devices (type, index, device)
-	// Note: parity disk for RAID4 is last normal disk, there is no '(P)'
-	raidlist->el[raidlist->entries].data_disks.entries = 0;
-	raidlist->el[raidlist->entries].spare_disks.entries = 0;
-	raidlist->el[raidlist->entries].failed_disks.entries = 0;
-	while((token = mr_strtok (string, delims, &lastpos))) {
-	  if ((pos = strstr(token, "("))) {
-	    type = *(pos+1);
-	  } else {
-	    type = ' ';
-	  }
-	  pos = strstr(token, "[");
-	  *pos = '\0';
-	  switch(type) {
-	  case ' ': // normal data disks
-	    raidlist->el[raidlist->entries].data_disks.el[raidlist->el[raidlist->entries].data_disks.entries].index = atoi(pos + 1);
-	    asprintf(&strtmp,"%s%s", device_prefix, token);
-	    strcpy(raidlist->el[raidlist->entries].data_disks.el[raidlist->el[raidlist->entries].data_disks.entries].device, strtmp);
-	    paranoid_free(strtmp);
-	    raidlist->el[raidlist->entries].data_disks.entries++;
-	    break;
-	  case 'S': // spare disks
-	    raidlist->el[raidlist->entries].spare_disks.el[raidlist->el[raidlist->entries].spare_disks.entries].index = atoi(pos + 1);
-	    asprintf(&strtmp,"%s%s", device_prefix, token);
-	    strcpy(raidlist->el[raidlist->entries].spare_disks.el[raidlist->el[raidlist->entries].spare_disks.entries].device, strtmp);
-	    paranoid_free(strtmp);
-	    raidlist->el[raidlist->entries].spare_disks.entries++;
-	    break;
-	  case 'F': // failed disks
-	    raidlist->el[raidlist->entries].failed_disks.el[raidlist->el[raidlist->entries].failed_disks.entries].index = atoi(pos + 1);
-	    asprintf(&strtmp,"%s%s", device_prefix, token);
-	    strcpy(raidlist->el[raidlist->entries].failed_disks.el[raidlist->el[raidlist->entries].failed_disks.entries].device, strtmp);
-	    paranoid_free(strtmp);
-	    raidlist->el[raidlist->entries].failed_disks.entries++;
-	    log_it("At least one failed disk found in RAID array.\n");
-	    break;
-	  default: // error
-	    log_msg(1, "Unknown device type '%c'\n", type);
-	    paranoid_free(string);
-	    paranoid_free(token);
-	    return 1;
-	    break;
-	  }
-	  paranoid_free(token);
-	}
+			// get RAID level
+			token = mr_strtok(string, delims, &lastpos);
+			if (token == NULL) {
+				// should not happen !
+				break;
+			}
+			// skip potential auto-read-only entry
+			if (!strcmp(token, "(auto-read-only)")) {
+				mr_free(token);
+				token = mr_strtok (string, delims, &lastpos);
+				if (token == NULL) {
+					// should not happen !
+					break;
+				}
+			}
+			if (!strcmp(token, "multipath")) {
+				raidlist->el[raidlist->entries].raid_level = -2;
+			} else if (!strcmp(token, "linear")) {
+				raidlist->el[raidlist->entries].raid_level = -1;
+			} else if (!strcmp(token, "raid0")) {
+				raidlist->el[raidlist->entries].raid_level = 0;
+			} else if (!strcmp(token, "raid1")) {
+				raidlist->el[raidlist->entries].raid_level = 1;
+			} else if (!strcmp(token, "raid4")) {
+				raidlist->el[raidlist->entries].raid_level = 4;
+			} else if (!strcmp(token, "raid5")) {
+				raidlist->el[raidlist->entries].raid_level = 5;
+			} else if (!strcmp(token, "raid6")) {
+				raidlist->el[raidlist->entries].raid_level = 6;
+			} else if (!strcmp(token, "raid10")) {
+				raidlist->el[raidlist->entries].raid_level = 10;
+			} else {
+				log_msg(1, "Unknown RAID level '%s'", token);
+				mr_free(string);
+				mr_free(token);
+				return 1;
+			}
+			mr_free(token);
 
-	// adjust index for each device so that it starts with 0 for every type
-	index_min = 99;
-	for (i=0; i<raidlist->el[raidlist->entries].data_disks.entries;i++) {
-	  if (raidlist->el[raidlist->entries].data_disks.el[i].index < index_min) {
-	    index_min = raidlist->el[raidlist->entries].data_disks.el[i].index;
-	  }
+			// get RAID devices (type, index, device)
+			// Note: parity disk for RAID4 is last normal disk, there is no '(P)'
+			raidlist->el[raidlist->entries].data_disks.entries = 0;
+			raidlist->el[raidlist->entries].spare_disks.entries = 0;
+			raidlist->el[raidlist->entries].failed_disks.entries = 0;
+			while((token = mr_strtok (string, delims, &lastpos))) {
+				if ((pos = strstr(token, "("))) {
+ 					type = *(pos+1);
+				} else {
+ 					type = ' ';
+				}
+				pos = strstr(token, "[");
+				*pos = '\0';
+				switch(type) {
+				case ' ': // normal data disks
+ 					raidlist->el[raidlist->entries].data_disks.el[raidlist->el[raidlist->entries].data_disks.entries].index = atoi(pos + 1);
+ 					mr_asprintf(strtmp,"%s%s", device_prefix, token);
+ 					strcpy(raidlist->el[raidlist->entries].data_disks.el[raidlist->el[raidlist->entries].data_disks.entries].device, strtmp);
+ 					mr_free(strtmp);
+ 					raidlist->el[raidlist->entries].data_disks.entries++;
+ 					break;
+				case 'S': // spare disks
+ 					raidlist->el[raidlist->entries].spare_disks.el[raidlist->el[raidlist->entries].spare_disks.entries].index = atoi(pos + 1);
+ 					mr_asprintf(strtmp,"%s%s", device_prefix, token);
+ 					strcpy(raidlist->el[raidlist->entries].spare_disks.el[raidlist->el[raidlist->entries].spare_disks.entries].device, strtmp);
+ 					mr_free(strtmp);
+ 					raidlist->el[raidlist->entries].spare_disks.entries++;
+ 					break;
+				case 'F': // failed disks
+ 					raidlist->el[raidlist->entries].failed_disks.el[raidlist->el[raidlist->entries].failed_disks.entries].index = atoi(pos + 1);
+ 					mr_asprintf(strtmp,"%s%s", device_prefix, token);
+ 					strcpy(raidlist->el[raidlist->entries].failed_disks.el[raidlist->el[raidlist->entries].failed_disks.entries].device, strtmp);
+ 					mr_free(strtmp);
+ 					raidlist->el[raidlist->entries].failed_disks.entries++;
+ 					log_it("At least one failed disk found in RAID array.\n");
+ 					break;
+				default: // error
+ 					log_msg(1, "Unknown device type '%c'", type);
+ 					mr_free(string);
+ 					mr_free(token);
+ 					return 1;
+ 					break;
+				}
+				mr_free(token);
+			}
+
+			// adjust index for each device so that it starts with 0 for every type
+			index_min = 99;
+			for (i=0; i<raidlist->el[raidlist->entries].data_disks.entries;i++) {
+				if (raidlist->el[raidlist->entries].data_disks.el[i].index < index_min) {
+					index_min = raidlist->el[raidlist->entries].data_disks.el[i].index;
+				}
+			}
+			if (index_min > 0) {
+				for (i=0; i<raidlist->el[raidlist->entries].data_disks.entries;i++) {
+ 					raidlist->el[raidlist->entries].data_disks.el[i].index = raidlist->el[raidlist->entries].data_disks.el[i].index - index_min;	
+				}
+			}
+			index_min = 99;
+			for (i=0; i<raidlist->el[raidlist->entries].spare_disks.entries;i++) {
+				if (raidlist->el[raidlist->entries].spare_disks.el[i].index < index_min) {
+					index_min = raidlist->el[raidlist->entries].spare_disks.el[i].index;
+				}
+			}
+			if (index_min > 0) {
+				for (i=0; i<raidlist->el[raidlist->entries].spare_disks.entries;i++) {
+						raidlist->el[raidlist->entries].spare_disks.el[i].index = raidlist->el[raidlist->entries].spare_disks.el[i].index - index_min;	
+				}
+			}
+			index_min = 99;
+			for (i=0; i<raidlist->el[raidlist->entries].failed_disks.entries;i++) {
+				if (raidlist->el[raidlist->entries].failed_disks.el[i].index < index_min) {
+					index_min = raidlist->el[raidlist->entries].failed_disks.el[i].index;
+				}
+			}
+			if (index_min > 0) {
+				for (i=0; i<raidlist->el[raidlist->entries].failed_disks.entries;i++) {
+						raidlist->el[raidlist->entries].failed_disks.el[i].index = raidlist->el[raidlist->entries].failed_disks.el[i].index - index_min;	
+				}
+			}
+			break;
+		case 2:  // config information
+			// check for persistent super block
+			if (strcasestr(string, "super non-persistent")) {
+				raidlist->el[raidlist->entries].persistent_superblock = 0;
+			} else {
+				raidlist->el[raidlist->entries].persistent_superblock = 1;
+			}
+			// extract chunk size
+			if (!(pos = strcasestr(string, "k chunk"))) {
+				raidlist->el[raidlist->entries].chunk_size = -1;
+			} else {
+				while (*pos != ' ') {
+					pos -= 1;
+					if (pos < string) {
+ 						log_it("String underflow!\n");
+ 						mr_free(string);
+ 						return 1;
+					}
+				}
+				raidlist->el[raidlist->entries].chunk_size = atoi(pos + 1);
+			}
+			// extract parity if present
+			if ((pos = strcasestr(string, "algorithm"))) {
+				raidlist->el[raidlist->entries].parity = atoi(pos + 9);
+			} else {
+				raidlist->el[raidlist->entries].parity = -1;
+			}
+			break;
+		case 3:  // optional build status information
+			if (!(pos = strchr(string, '\%'))) {
+				if (strcasestr(string, "delayed")) {
+					raidlist->el[raidlist->entries].progress = -1;	// delayed (therefore, stuck at 0%)
+				} else {
+					raidlist->el[raidlist->entries].progress = 999;	// not found
+				}
+			} else {
+				while (*pos != ' ') {
+					pos -= 1;
+					if (pos < string) {
+ 						printf("ERROR: String underflow!\n");
+						mr_free(string);
+						return 1;
+					}
+				}
+				raidlist->el[raidlist->entries].progress = atoi(pos);
+			}
+			break;
+		default: // error or IN PROGRESS
+			if (raidlist->el[raidlist->entries].progress != -1 &&
+				raidlist->el[raidlist->entries].progress != 999) {
+				log_msg(1, "Row %d should not occur in record!", row);
+			}
+			break;
+		}
+	row++;
 	}
-	if (index_min > 0) {
-	  for (i=0; i<raidlist->el[raidlist->entries].data_disks.entries;i++) {
-	    raidlist->el[raidlist->entries].data_disks.el[i].index = raidlist->el[raidlist->entries].data_disks.el[i].index - index_min;	
-	  }
-	}
-	index_min = 99;
-	for (i=0; i<raidlist->el[raidlist->entries].spare_disks.entries;i++) {
-	  if (raidlist->el[raidlist->entries].spare_disks.el[i].index < index_min) {
-	    index_min = raidlist->el[raidlist->entries].spare_disks.el[i].index;
-	  }
-	}
-	if (index_min > 0) {
-	  for (i=0; i<raidlist->el[raidlist->entries].spare_disks.entries;i++) {
-	    raidlist->el[raidlist->entries].spare_disks.el[i].index = raidlist->el[raidlist->entries].spare_disks.el[i].index - index_min;	
-	  }
-	}
-	index_min = 99;
-	for (i=0; i<raidlist->el[raidlist->entries].failed_disks.entries;i++) {
-	  if (raidlist->el[raidlist->entries].failed_disks.el[i].index < index_min) {
-	    index_min = raidlist->el[raidlist->entries].failed_disks.el[i].index;
-	  }
-	}
-	if (index_min > 0) {
-	  for (i=0; i<raidlist->el[raidlist->entries].failed_disks.entries;i++) {
-	    raidlist->el[raidlist->entries].failed_disks.el[i].index = raidlist->el[raidlist->entries].failed_disks.el[i].index - index_min;	
-	  }
-	}
-	break;
-      case 2:  // config information
-	// check for persistent super block
-	if (strcasestr(string, "super non-persistent")) {
-	  raidlist->el[raidlist->entries].persistent_superblock = 0;
-	} else {
-	  raidlist->el[raidlist->entries].persistent_superblock = 1;
-	}
-	// extract chunk size
-	if (!(pos = strcasestr(string, "k chunk"))) {
-	  raidlist->el[raidlist->entries].chunk_size = -1;
-	} else {
-	  while (*pos != ' ') {
-	    pos -= 1;
-	    if (pos < string) {
-	      log_it("String underflow!\n");
-	      paranoid_free(string);
-	      return 1;
-	    }
-	  }
-	  raidlist->el[raidlist->entries].chunk_size = atoi(pos + 1);
-	}
-	// extract parity if present
-	if ((pos = strcasestr(string, "algorithm"))) {
-	  raidlist->el[raidlist->entries].parity = atoi(pos + 9);
-	} else {
-	  raidlist->el[raidlist->entries].parity = -1;
-	}
-	break;
-      case 3:  // optional build status information
-	if (!(pos = strchr(string, '\%'))) {
-	  if (strcasestr(string, "delayed")) {
-	    raidlist->el[raidlist->entries].progress = -1;	// delayed (therefore, stuck at 0%)
-	  } else {
-	    raidlist->el[raidlist->entries].progress = 999;	// not found
-	  }
-	} else {
-	  while (*pos != ' ') {
-	    pos -= 1;
-	    if (pos < string) {
-	      printf("ERROR: String underflow!\n");
-	      paranoid_free(string);
-	      return 1;
-	    }
-	  }
-	  raidlist->el[raidlist->entries].progress = atoi(pos);
-	}
-	break;
-      default: // error or IN PROGRESS
-	if (raidlist->el[raidlist->entries].progress != -1 &&
-		raidlist->el[raidlist->entries].progress != 999) {
-		log_msg(1, "Row %d should not occur in record!\n", row);
-	}
-	break;
-      }
-      row++;
-    }
-  }
-  // close file
-  fclose(fin);
-  // free string
-  paranoid_free(string);
-  // return success
-  return 0;
+	// free string
+	mr_free(string);
+}
+// close file
+fclose(fin);
+// return success
+return 0;
 
 }
 
 
 
 
-int create_raidtab_from_mdstat(char *raidtab_fname)
+int create_raidtab_from_mdstat(char *mdstat_fname,char *raidtab_fname)
 {
 	struct raidlist_itself *raidlist;
 	int retval = 0;
@@ -1264,8 +1235,8 @@ int create_raidtab_from_mdstat(char *raidtab_fname)
 	raidlist = malloc(sizeof(struct raidlist_itself));
 
 	// FIXME: Prefix '/dev/' should really be dynamic!
-	if (parse_mdstat(raidlist, "/dev/")) {
-		log_to_screen("Sorry, cannot read %s", MDSTAT_FILE);
+	if (parse_mdstat(mdstat_fname,raidlist, "/dev/")) {
+		log_to_screen("Sorry, cannot read %s", mdstat_fname);
 		return (1);
 	}
 
