@@ -1,5 +1,5 @@
 /* libmondo-stream.c
-   $Id: libmondo-stream.c 3150 2013-06-21 04:48:23Z bruno $
+   $Id: libmondo-stream.c 3527 2016-03-04 05:47:07Z bruno $
 
 ...tools for talking to tapes, Monitas streams, etc.
 
@@ -28,7 +28,7 @@
 #define STR_HEADER	"Mondolicious, baby"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-stream.c 3150 2013-06-21 04:48:23Z bruno $";
+//static char cvsid[] = "$Id: libmondo-stream.c 3527 2016-03-04 05:47:07Z bruno $";
 extern bool g_sigpipe;
 extern int g_tape_buffer_size_MB;
 
@@ -137,16 +137,10 @@ int closein_tape()
 		}
 	}
 	sleep(1);
-	paranoid_system("sync");
+	sync();
 	sleep(1);
 	paranoid_pclose(g_tape_stream);
 	log_it("closein_tape() -- leaving");
-/*
-  for(i=0; i < g_tapecatalog->entries; i++)
-    {
-      log_it("i=%d type=%s num=%d aux=%ld", i, (g_tapecatalog->el[i].type==fileset)?"fileset":"bigslice", g_tapecatalog->el[i].number, g_tapecatalog->el[i].aux);
-    }
-*/
 	if (!bkpinfo->please_dont_eject) {
 		eject_device(bkpinfo->media_device);
 	}
@@ -182,7 +176,7 @@ int closeout_tape()
 	blk = (char *) malloc(256 * 1024);
 
 	sleep(1);
-	paranoid_system("sync");
+	sync();
 	sleep(1);
 	log_it("closeout_tape() -- entering");
 	retval +=
@@ -227,8 +221,6 @@ int closeout_tape()
 			   g_tapecatalog->el[i].number, g_tapecatalog->el[i].aux,
 			   g_tapecatalog->el[i].tape_posK);
 	}
-	//  if (!bkpinfo->please_dont_eject)
-	//    { eject_device(bkpinfo->media_device); }
 	paranoid_free(blk);
 	paranoid_free(g_tapecatalog);
 	return (retval);
@@ -241,7 +233,7 @@ bool mt_says_tape_exists(char *dev)
 	char *command = NULL;
 	int res;
 
-	mr_asprintf(&command, "mt -f %s status", dev);
+	mr_asprintf(command, "mt -f %s status", dev);
 	res = run_program_and_log_output(command, 1);
 	mr_free(command);
 
@@ -265,46 +257,50 @@ int find_tape_device_and_size(char *dev, char *siz)
 {
 	char tmp[MAX_STR_LEN];
 	char *command = NULL;
-	char cdr_exe[MAX_STR_LEN];
+	char *cdr_exe = NULL;
 	int res;
 
 	log_to_screen("I am looking for your tape streamer. Please wait.");
 	dev[0] = siz[0] = '\0';
 	if (find_home_of_exe("cdrecord")) {
-		strcpy(cdr_exe, "cdrecord");
+		mr_asprintf(cdr_exe, "cdrecord");
 	} else {
-		strcpy(cdr_exe, "dvdrecord");
+		mr_asprintf(cdr_exe, "dvdrecord");
 	}
-	mr_asprintf(&command, "%s -scanbus 2> /dev/null | grep -i tape | wc -l",
-			cdr_exe);
-	strcpy(tmp, call_program_and_get_last_line_of_output(command));
+	mr_asprintf(command, "%s -scanbus 2> /dev/null | grep -i tape | wc -l", cdr_exe);
+	mr_asprintf(tmp, "%s", call_program_and_get_last_line_of_output(command));
 	mr_free(command);
 
 	if (atoi(tmp) != 1) {
-		log_it
-			("Either too few or too many tape streamers for me to detect...");
+		log_it("Either too few or too many tape streamers for me to detect...");
 		strcpy(dev, VANILLA_SCSI_TAPE);
+		mr_free(tmp);
+		mr_free(cdr_exe);
 		return 1;
 	}
-	mr_asprintf(&command,
-			"%s -scanbus 2> /dev/null | tr -s '\t' ' ' | grep \"[0-9]*,[0-9]*,[0-9]*\" | grep -v \"[0-9]*) \\*\" | grep -i TAPE | cut -d' ' -f2 | head -n1",
-			cdr_exe);
-	strcpy(tmp, call_program_and_get_last_line_of_output(command));
+	mr_free(tmp);
+
+	mr_asprintf(command, "%s -scanbus 2> /dev/null | tr -s '\t' ' ' | grep \"[0-9]*,[0-9]*,[0-9]*\" | grep -v \"[0-9]*) \\*\" | grep -i TAPE | cut -d' ' -f2 | head -n1", cdr_exe);
+	mr_asprintf(tmp, "%s", call_program_and_get_last_line_of_output(command));
 	mr_free(command);
 
 	if (strlen(tmp) < 2) {
 		log_it("Could not find tape device");
+		mr_free(cdr_exe);
+		mr_free(tmp);
 		return 1;
 	}
-	mr_asprintf(&command,
-			"%s -scanbus 2> /dev/null | tr -s '\t' ' ' | grep \"[0-9]*,[0-9]*,[0-9]*\" | grep -v \"[0-9]*) \\*\" | grep -i TAPE | cut -d' ' -f3 | cut -d')' -f1 | head -n1",
-			cdr_exe);
-	strcpy(tmp, call_program_and_get_last_line_of_output(command));
+	mr_asprintf(command, "%s -scanbus 2> /dev/null | tr -s '\t' ' ' | grep \"[0-9]*,[0-9]*,[0-9]*\" | grep -v \"[0-9]*) \\*\" | grep -i TAPE | cut -d' ' -f3 | cut -d')' -f1 | head -n1", cdr_exe);
+	mr_free(cdr_exe);
+
+	mr_asprintf(tmp, "%s", call_program_and_get_last_line_of_output(command));
 	mr_free(command);
 
 	strcpy(dev, VANILLA_SCSI_TAPE);
 	dev[strlen(dev) - 1] = '\0';
 	strcat(dev, tmp);			// e.g. '/dev/st0' becomes '/dev/stN'
+	mr_free(tmp);
+
 	res = 0;
 	if (!mt_says_tape_exists(dev)) {
 		strcpy(dev, ALT_TAPE);
@@ -323,18 +319,20 @@ int find_tape_device_and_size(char *dev, char *siz)
 		}
 	}
 
-	log_it("At this point, dev = %s and res = %d", dev, res);
+	if (dev) {
+		log_it("At this point, dev = %s and res = %d", dev, res);
+	} else {
+		log_it("At this point, dev is NULL and res = %d", dev, res);
+	}
 
-	strcpy(tmp, call_program_and_get_last_line_of_output("\
-cdrecord -scanbus 2> /dev/null | tr -s '\t' ' ' | \
-grep \"[0-9]*,[0-9]*,[0-9]*\" | grep -v \"[0-9]*) \\*\" | grep -i TAPE | \
-awk '{for(i=1; i<NF; i++) { if (index($i, \"GB\")>0) { print $i;};};};'"));
+	mr_asprintf(tmp, "%s", call_program_and_get_last_line_of_output("cdrecord -scanbus 2> /dev/null | tr -s '\t' ' ' | grep \"[0-9]*,[0-9]*,[0-9]*\" | grep -v \"[0-9]*) \\*\" | grep -i TAPE | awk '{for(i=1; i<NF; i++) { if (index($i, \"GB\")>0) { print $i;};};};'"));
 
 	if (mt_says_tape_exists(dev)) {
 		res = 0;
 	} else {
 		log_it("Turning %s", dev);
-		strcpy(tmp, (strrchr(dev, '/') != NULL) ? strrchr(dev, '/') : dev);
+		mr_free(tmp);
+		mr_asprintf(tmp, "%s", (strrchr(dev, '/') != NULL) ? strrchr(dev, '/') : dev);
 		sprintf(dev, "/dev/os%s", tmp);
 		log_it("...into %s", dev);
 		if (mt_says_tape_exists(dev)) {
@@ -354,18 +352,14 @@ awk '{for(i=1; i<NF; i++) { if (index($i, \"GB\")>0) { print $i;};};};'"));
 	if (strlen(tmp) < 2) {
 		siz[0] = '\0';
 		log_it("Warning - size of tape unknown");
-		return (0);
 	} else {
 		strcpy(siz, tmp);
-		return (0);
 	}
+	mr_free(tmp);
+	return (0);
 }
 
 
-
-
-
-
 int read_EXAT_files_from_tape(long long *ptmp_size, char *tmp_fname,
 							  int *pctrl_chr, char *xattr_fname,
 							  char *acl_fname)
@@ -382,7 +376,7 @@ int read_EXAT_files_from_tape(long long *ptmp_size, char *tmp_fname,
 			wrong_marker(BLK_START_EXAT_FILE, *pctrl_chr);
 		}
 		if (strstr(fname, "xattr") == NULL) {
-			mr_asprintf(&tmp,"Wrong order expected xattr, got %s, sunshine.", fname);
+			mr_asprintf(tmp,"Wrong order expected xattr, got %s, sunshine.", fname);
 			fatal_error(tmp);
 		}
 		read_file_from_stream_to_file(xattr_fname, *ptmp_size);
@@ -412,7 +406,7 @@ int read_EXAT_files_from_tape(long long *ptmp_size, char *tmp_fname,
 			wrong_marker(BLK_START_EXAT_FILE, *pctrl_chr);
 		}
 		if (strstr(fname, "acl") == NULL) {
-			mr_asprintf(&tmp,"Wrong order expected acl, got %s, sunshine.", fname);
+			mr_asprintf(tmp,"Wrong order expected acl, got %s, sunshine.", fname);
 			fatal_error(tmp);
 		}
 		read_file_from_stream_to_file(acl_fname, *ptmp_size);
@@ -478,9 +472,7 @@ void insist_on_this_tape_number(int tapeno)
 
 	log_it("Insisting on tape #%d", tapeno);
 	if (g_current_media_number != tapeno) {
-		mr_asprintf(&tmp,
-				"When the tape drive goes quiet, please insert volume %d in this series.",
-				tapeno);
+		mr_asprintf(tmp, "When the tape drive goes quiet, please insert volume %d in this series.", tapeno);
 		popup_and_OK(tmp);
 		mr_free(tmp);
 		open_evalcall_form("Waiting while the tape drive settles");
@@ -548,9 +540,9 @@ int maintain_collection_of_recent_archives(char *td, char *latest_fname)
 	} else {
 		suffix[0] = '\0';
 	}
-	mr_asprintf(&tmpdir, "%s/tmpfs/backcatalog", td);
+	mr_asprintf(tmpdir, "%s/tmpfs/backcatalog", td);
 	mkdir(tmpdir, 0x700);
-	mr_asprintf(&command, "cp -f %s %s", latest_fname, tmpdir);
+	mr_asprintf(command, "cp -f %s %s", latest_fname, tmpdir);
 	if (run_program_and_log_output(command, 6)) {
 		log_it("Warning - failed to copy %s to backcatalog at %s",
 			   latest_fname, tmpdir);
@@ -578,7 +570,7 @@ int maintain_collection_of_recent_archives(char *td, char *latest_fname)
 	}
 
 	for (i = curr - 1; i >= 0 && curr - i < 10; i--) {
-		mr_asprintf(&old_fname, "%s/%s", tmpdir, g_tapecatalog->el[i].fname);
+		mr_asprintf(old_fname, "%s/%s", tmpdir, g_tapecatalog->el[i].fname);
 		unlink(old_fname);
 		mr_free(old_fname);
 	}
@@ -613,12 +605,15 @@ int set_tape_block_size_with_mt(long internal_tape_block_size)
 	char *tmp = NULL;
 	int res;
 
+	if (bkpinfo->media_device == NULL) {
+		return(1);
+	}
+
 	if (strncmp(bkpinfo->media_device, "/dev/", 5)) {
-		log_msg(1,
-				"Not using 'mt setblk'. This isn't an actual /dev entry.");
+		log_msg(1, "Not using 'mt setblk'. This isn't an actual /dev entry.");
 		return (0);
 	}
-	mr_asprintf(&tmp, "mt -f %s setblk %ld", bkpinfo->media_device, internal_tape_block_size);
+	mr_asprintf(tmp, "mt -f %s setblk %ld", bkpinfo->media_device, internal_tape_block_size);
 	res = run_program_and_log_output(tmp, 3);
 	mr_free(tmp);
 	return (res);
@@ -686,12 +681,16 @@ int skip_obdr(void)
 	char *command = NULL;
 	int res = 0;
 
+	if (bkpinfo->media_device == NULL) {
+		return(1);
+	}
+
 	log_it("Skipping OBDR headers");
-	mr_asprintf(&command, "mt -f %s rewind",bkpinfo->media_device);
+	mr_asprintf(command, "mt -f %s rewind",bkpinfo->media_device);
 	res = run_program_and_log_output(command, 1);
 	paranoid_free(command);
 
-	mr_asprintf(&command, "mt -f %s fsf 2",bkpinfo->media_device);
+	mr_asprintf(command, "mt -f %s fsf 2",bkpinfo->media_device);
 	res = run_program_and_log_output(command, 1);
 	paranoid_free(command);
 
@@ -712,33 +711,37 @@ int create_obdr(void)
 	char *command = NULL;
 	int res = 0;
 
+	if (bkpinfo->media_device == NULL) {
+		return(1);
+	}
+
 	log_it("Creating OBDR headers");
 	/* OBDR: First block 10 kB of zero bs = 512 */
-	mr_asprintf(&command, "mt -f %s compression off",bkpinfo->media_device);
+	mr_asprintf(command, "mt -f %s compression off",bkpinfo->media_device);
 	res = run_program_and_log_output(command, 1);
 	paranoid_free(command);
 
-	mr_asprintf(&command, "mt -f %s rewind",bkpinfo->media_device);
+	mr_asprintf(command, "mt -f %s rewind",bkpinfo->media_device);
 	res += run_program_and_log_output(command, 1);
 	paranoid_free(command);
 
 	set_tape_block_size_with_mt(512);
 
-	mr_asprintf(&command, "dd if=/dev/zero of=%s bs=512 count=20",bkpinfo->media_device);
+	mr_asprintf(command, "dd if=/dev/zero of=%s bs=512 count=20",bkpinfo->media_device);
 	res += run_program_and_log_output(command, 1);
 	paranoid_free(command);
 
 	/* OBDR: then ISO boot image bs = 2048 */
 	set_tape_block_size_with_mt(2048);
 
-	mr_asprintf(&command, "dd if=%s of=%s bs=2048",MINDI_CACHE"/mondorescue.iso",bkpinfo->media_device);
+	mr_asprintf(command, "dd if=%s of=%s bs=2048",MINDI_CACHE"/mindi.iso",bkpinfo->media_device);
 	res += run_program_and_log_output(command, 1);
 	paranoid_free(command);
 
 	set_tape_block_size_with_mt(bkpinfo->internal_tape_block_size);
 
 	/* restore compression mode on */
-	mr_asprintf(&command, "mt -f %s compression on",bkpinfo->media_device);
+	mr_asprintf(command, "mt -f %s compression on",bkpinfo->media_device);
 	res = run_program_and_log_output(command, 1);
 	paranoid_free(command);
 
@@ -779,10 +782,7 @@ int openin_tape()
 
 	/*@ end vars *************************************************** */
 
-	assert_string_is_neither_NULL_nor_zerolength(bkpinfo->media_device);
-	if (!(g_tapecatalog = malloc(sizeof(struct s_tapecatalog)))) {
-		fatal_error("Cannot alloc mem for tape catalog");
-	}
+	g_tapecatalog = mr_malloc(sizeof(struct s_tapecatalog));
 	g_tapecatalog->entries = 0;
 	g_tape_posK = 0;
 	if (g_tape_stream) {
@@ -799,20 +799,17 @@ int openin_tape()
 	} else {
 		if (bkpinfo->media_device == NULL) {
 			log_it("Not able to skip OBDR - Restore will have to be done manually");
+			return (1);
 		}
 		set_tape_block_size_with_mt(bkpinfo->internal_tape_block_size);
 	}
 
 	insist_on_this_tape_number(1);
-	mr_asprintf(&outfname, "%s/tmp/all.tar.gz", bkpinfo->tmpdir);
+	mr_asprintf(outfname, "%s/tmp/all.tar.gz", bkpinfo->tmpdir);
 	make_hole_for_file(outfname);
 
-//  start_buffer_process( bkpinfo->media_device, g_tape_fifo, FALSE);
 	log_it("Opening IN tape");
-	if (!
-		(g_tape_stream =
-		 open_device_via_buffer(bkpinfo->media_device, 'r',
-								bkpinfo->internal_tape_block_size))) {
+	if (!(g_tape_stream = open_device_via_buffer(bkpinfo->media_device, 'r', bkpinfo->internal_tape_block_size))) {
 		log_OS_error(g_tape_fifo);
 		log_to_screen("Cannot openin stream device");
 		mr_free(outfname);
@@ -820,7 +817,7 @@ int openin_tape()
 	}
 	log_to_screen("Reading stream");
 	log_it("stream device = '%s'", bkpinfo->media_device);
-/* skip data disks */
+	/* skip data disks */
 	open_evalcall_form("Skipping data disks on stream");
 	log_to_screen("Skipping data disks on stream");
 	if (!(fout = fopen(outfname, "w"))) {
@@ -873,12 +870,11 @@ int openin_tape()
 	if (chdir(bkpinfo->tmpdir)) {
 		// FIXME
 	}
-	mr_asprintf(&tmp, "tar -zxf %s ./tmp/mondo-restore.cfg 2> /dev/null",
-			outfname);
+	mr_asprintf(tmp, "tar -zxf %s ./tmp/mondorestore.cfg 2> /dev/null", outfname);
 	paranoid_system(tmp);
 	mr_free(tmp);
 
-	paranoid_system("cp -f tmp/mondo-restore.cfg . 2> /dev/null");
+	paranoid_system("cp -f tmp/mondorestore.cfg . 2> /dev/null");
 	if (chdir(old_cwd)) {
 		// FIXME
 	}
@@ -902,8 +898,12 @@ int openout_cdstream(char *cddev, int speed)
 
 	/*@ end vars *************************************************** */
 
+	if (cddev == NULL) {
+		log_to_screen("Failed to openout NULL cddev");
+		return (1);
+	}
 	/*  add 'dummy' if testing */
-	mr_asprintf(&command, "cdrecord -eject dev=%s speed=%d fs=24m -waiti - >> %s 2>> %s", cddev, speed, MONDO_LOGFILE, MONDO_LOGFILE);
+	mr_asprintf(command, "cdrecord -eject dev=%s speed=%d fs=24m -waiti - >> %s 2>> %s", cddev, speed, MONDO_LOGFILE, MONDO_LOGFILE);
 	/*  initialise the catalog */
 	g_current_media_number = 1;
 	if (!(g_tapecatalog = malloc(sizeof(struct s_tapecatalog)))) {
@@ -941,9 +941,11 @@ if (g_tape_stream) {
 	log_it("FYI - I won't 'openout' the tape. It's already open.");
 	return (0);
 }
-if (!(g_tapecatalog = malloc(sizeof(struct s_tapecatalog)))) {
-	fatal_error("Cannot alloc mem for tape catalog");
+if (bkpinfo->media_device == NULL) {
+	log_it("Unable to openout NULL device");
+	return(1);
 }
+g_tapecatalog = mr_malloc(sizeof(struct s_tapecatalog));
 g_tapecatalog->entries = 0;
 g_tape_posK = 0;
 
@@ -953,9 +955,7 @@ if (bkpinfo->use_obdr) {
 	set_tape_block_size_with_mt(bkpinfo->internal_tape_block_size);
 }
 log_it("Opening OUT tape");
-if (!
-	(g_tape_stream =
-	 open_device_via_buffer(bkpinfo->media_device, 'w', bkpinfo->internal_tape_block_size))) {
+if (!(g_tape_stream = open_device_via_buffer(bkpinfo->media_device, 'w', bkpinfo->internal_tape_block_size))) {
 	log_OS_error(g_tape_fifo);
 	log_to_screen("Cannot openin stream device");
 	return (1);
@@ -968,7 +968,6 @@ return (0);
 
 /**
  * Copy a file from the opened stream (CD or tape) to @p outfile.
- * @param bkpinfo The backup information structure. @c bkpinfo->media_device is the only field used.
  * @param outfile The file to write to.
  * @param size The size of the file in the input stream.
  * @return 0 for success, nonzero for failure.
@@ -1030,12 +1029,10 @@ int
 read_file_from_stream_FULL(char *outfname, FILE * foutstream, long long orig_size)
 {
 	/*@ buffers ***************************************************** */
-	char *tmp = NULL;
 	char *datablock;
 	char *temp_fname;
 	char *temp_cksum;
 	char *actual_cksum = NULL;
-//  char *pA, *pB;
 
 	/*@ int ********************************************************* */
 	int retval = 0;
@@ -1077,9 +1074,7 @@ read_file_from_stream_FULL(char *outfname, FILE * foutstream, long long orig_siz
 		// FIXME
 	}
 	if (orig_size != temp_size && orig_size != -1) {
-		mr_asprintf(&tmp, "output file's size should be %ld K but is apparently %ld K", (long) size >> 10, (long) temp_size >> 10);
-		log_to_screen(tmp);
-		mr_free(tmp);
+		log_to_screen("output file's size should be %ld K but is apparently %ld K", (long) size >> 10, (long) temp_size >> 10);
 	}
 	if (ctrl_chr != BLK_START_FILE) {
 		wrong_marker(BLK_START_FILE, ctrl_chr);
@@ -1140,7 +1135,7 @@ read_file_from_stream_FULL(char *outfname, FILE * foutstream, long long orig_siz
 			total_read_from_tape_for_this_file);
 	log_msg(6, ".......................... Should be %lld", orig_size);
 	g_tape_posK += total_read_from_tape_for_this_file / 1024;
-	mr_asprintf(&actual_cksum, "%04x%04x", crc16, crctt);
+	mr_asprintf(actual_cksum, "%04x%04x", crc16, crctt);
 	if (foutstream) {			/*log_it("Finished writing to foutstream"); */
 	} else {
 		paranoid_fclose(fout);
@@ -1150,16 +1145,8 @@ read_file_from_stream_FULL(char *outfname, FILE * foutstream, long long orig_siz
 		wrong_marker(BLK_STOP_FILE, ctrl_chr);
 	}
 	if (strcmp(temp_cksum, actual_cksum)) {
-		mr_asprintf(&tmp, "actual cksum=%s; recorded cksum=%s", actual_cksum,
-				temp_cksum);
-		log_to_screen(tmp);
-		mr_free(tmp);
-
-		mr_asprintf(&tmp, "%s (%ld K) is corrupt on tape", temp_fname,
-				(long) orig_size >> 10);
-		log_to_screen(tmp);
-		mr_free(tmp);
-
+		log_to_screen("actual cksum=%s; recorded cksum=%s", actual_cksum, temp_cksum);
+		log_to_screen("%s (%ld K) is corrupt on tape", temp_fname, (long) orig_size >> 10);
 		retval++;
 	}
 	mr_free(actual_cksum);
@@ -1203,18 +1190,11 @@ read_header_block_from_stream(long long *plen, char *filename,
 	while (!(*pcontrol_char = tempblock[7000])) {
 		g_tape_posK += fread(tempblock, 1, (size_t) TAPE_BLOCK_SIZE, g_tape_stream) / 1024;
 	}
-/*  memcpy((char*)plength_of_incoming_file,(char*)tempblock+7001,sizeof(long long)); */
-/*  for(*plen=0,i=7;i>=0;i--) {*plen<<=8; *plen |= tempblock[7001+i];} */
 	memcpy((char *) plen, tempblock + 7001, sizeof(long long));
 	if (strcmp(tempblock + 6000 + *pcontrol_char, STR_HEADER)) {
 		log_it("Bad header block at %ld K", (long) g_tape_posK);
 	}
 	strcpy(filename, tempblock + 1000);
-/*  strcpy(cksum,tempblock+5555);*/
-/*  log_it( "%s  (reading) fname=%s, filesize=%ld K",
-	   marker_to_string (*pcontrol_char), filename,
-	   (long) ((*plen) >> 10));
-*/
 	if (*pcontrol_char == BLK_ABORTED_BACKUP) {
 		log_to_screen("I can't verify an aborted backup.");
 		retval = 1;
@@ -1290,7 +1270,7 @@ int register_in_tape_catalog(t_archtype type, int number, long aux,
  * autodetecting the size. Huh?
  */
 
-/* BERLIOS: Should be reviewed for mediasize being a off_t ??? */
+/* TODO: Should be reviewed for mediasize being a off_t ??? */
 bool
 should_we_write_to_next_tape(long mediasize,
 							 off_t length_of_incoming_file)
@@ -1362,6 +1342,9 @@ int skip_incoming_files_until_we_find_this_one(char
 	}
 	while (ctrl_chr != BLK_START_FILE) {
 		res = read_header_block_from_stream(&temp_size, temp_fname, &ctrl_chr);
+		if (res) {
+			//FIXME
+		}
 		if (ctrl_chr == BLK_START_FILE) {
 			break;
 		}
@@ -1382,8 +1365,7 @@ int skip_incoming_files_until_we_find_this_one(char
 		pB = the_file_I_was_reading;
 	}
 	while (strcmp(pA, pB)) {
-		log_msg(6, "Skipping %s (it's not %s)", temp_fname,
-				the_file_I_was_reading);
+		log_msg(6, "Skipping %s (it's not %s)", temp_fname, the_file_I_was_reading);
 		for (size = temp_size; size > 0; size -= bytes_to_write) {
 			bytes_to_write =
 				(size < TAPE_BLOCK_SIZE) ? (long) size : TAPE_BLOCK_SIZE;
@@ -1420,8 +1402,7 @@ int skip_incoming_files_until_we_find_this_one(char
 			pB = the_file_I_was_reading;
 		}
 	}
-	log_msg(2, "Reading %s (it matches %s)", temp_fname,
-			the_file_I_was_reading);
+	log_msg(2, "Reading %s (it matches %s)", temp_fname, the_file_I_was_reading);
 	paranoid_free(temp_fname);
 	paranoid_free(datablock);
 	return (0);
@@ -1443,6 +1424,11 @@ int start_to_read_from_next_tape()
 	malloc_string(sz_msg);
 	/*@ end vars *************************************************** */
 
+	if (bkpinfo->media_device == NULL) {
+		log_it("Unable to open in from NULL device");
+		return (1);
+	}
+
 	paranoid_pclose(g_tape_stream);
 	sync();
 	sync();
@@ -1450,10 +1436,7 @@ int start_to_read_from_next_tape()
 	log_it("Next tape requested.");
 	insist_on_this_tape_number(g_current_media_number + 1);	// will increment it, too
 	log_it("Opening IN the next tape");
-	if (!
-		(g_tape_stream =
-		 open_device_via_buffer(bkpinfo->media_device, 'r',
-								bkpinfo->internal_tape_block_size))) {
+	if (!(g_tape_stream = open_device_via_buffer(bkpinfo->media_device, 'r', bkpinfo->internal_tape_block_size))) {
 		log_OS_error(g_tape_fifo);
 		log_to_screen("Cannot openin stream device");
 		return (1);
@@ -1486,17 +1469,19 @@ int start_to_write_to_next_tape()
 	int res = 0;
 	char *command = NULL;
 
+	if (bkpinfo->media_device == NULL) {
+		log_it("Unable to open out from NULL device");
+		return (1);
+	}
+
 	paranoid_pclose(g_tape_stream);
-	paranoid_system("sync");
-	paranoid_system("sync");
-	paranoid_system("sync");
+	sync();
+	sync();
+	sync();
 	log_it("New tape requested.");
 	insist_on_this_tape_number(g_current_media_number + 1);	// will increment g_current_media, too
 	if (bkpinfo->backup_media_type == cdstream) {
-		mr_asprintf(&command,
-				"cdrecord -eject dev=%s speed=%d fs=24m -waiti - >> %s 2>> %s",
-				bkpinfo->media_device, bkpinfo->cdrw_speed, MONDO_LOGFILE,
-				MONDO_LOGFILE);
+		mr_asprintf(command, "cdrecord -eject dev=%s speed=%d fs=24m -waiti - >> %s 2>> %s", bkpinfo->media_device, bkpinfo->cdrw_speed, MONDO_LOGFILE, MONDO_LOGFILE);
 		log_it("Opening OUT to next CD with the command");
 		log_it(command);
 		log_it("Let's see what happens, shall we?");
@@ -1509,10 +1494,7 @@ int start_to_write_to_next_tape()
 		}
 	} else {
 		log_it("Opening OUT to next tape");
-		if (!
-			(g_tape_stream =
-			 open_device_via_buffer(bkpinfo->media_device, 'w',
-									bkpinfo->internal_tape_block_size))) {
+		if (!(g_tape_stream = open_device_via_buffer(bkpinfo->media_device, 'w', bkpinfo->internal_tape_block_size))) {
 			log_OS_error(g_tape_fifo);
 			log_to_screen("Cannot openin stream device");
 			return (1);
@@ -1548,7 +1530,7 @@ int write_backcatalog_to_tape()
 	log_msg(2, "I am now writing back catalog to tape");
 	last = g_tapecatalog->entries - 1;
 	for (i = 0; i <= last; i++) {
-		mr_asprintf(&fname, "%s/tmpfs/backcatalog/%s", bkpinfo->tmpdir, g_tapecatalog->el[i].fname);
+		mr_asprintf(fname, "%s/tmpfs/backcatalog/%s", bkpinfo->tmpdir, g_tapecatalog->el[i].fname);
 		if (!does_file_exist(fname)) {
 			log_msg(6, "Can't write %s - it doesn't exist.", fname);
 		} else {
@@ -1583,7 +1565,6 @@ int write_data_disks_to_stream(char *fname)
 {
 	/*@ pointers *************************************************** */
 	FILE *fin;
-	char *tmp = NULL;
 
 	/*@ long ******************************************************* */
 	long m = -1;
@@ -1601,9 +1582,7 @@ int write_data_disks_to_stream(char *fname)
 	log_to_screen("Writing data disks to tape");
 	log_it("Data disks = %s", fname);
 	if (!does_file_exist(fname)) {
-		mr_asprintf(&tmp, "Cannot find %s", fname);
-		log_to_screen(tmp);
-		mr_free(tmp);
+		log_to_screen("Cannot find %s", fname);
 		return (1);
 	}
 	if (!(fin = fopen(fname, "r"))) {
@@ -1646,7 +1625,6 @@ int write_data_disks_to_stream(char *fname)
 int write_file_to_stream_from_file(char *infile)
 {
 	/*@ buffers **************************************************** */
-	char *tmp = NULL;
 	char datablock[TAPE_BLOCK_SIZE];
 	char *checksum = NULL;
 	char *infile_basename;
@@ -1699,10 +1677,7 @@ int write_file_to_stream_from_file(char *infile)
 	} else {
 		p++;
 	}
-	mr_asprintf(&tmp, "Writing file '%s' to tape (%ld KB)", p,
-			(long) filesize >> 10);
-	log_it(tmp);
-	mr_free(tmp);
+	log_it("Writing file '%s' to tape (%ld KB)", p, (long) filesize >> 10);
 	write_header_block_to_stream(filesize, infile_basename,
 								 BLK_START_FILE);
 //go_here_to_restart_saving_of_file:
@@ -1744,8 +1719,8 @@ int write_file_to_stream_from_file(char *infile)
 #endif
 	}
 	paranoid_fclose(fin);
-	mr_asprintf(&checksum, "%04x%04x", crc16, crctt);
-	/* BERLIOS: what does it do ??? */
+	mr_asprintf(checksum, "%04x%04x", crc16, crctt);
+	/* TODO: what does it do ??? */
 	write_header_block_to_stream((off_t)g_current_media_number, checksum, BLK_STOP_FILE);
 	mr_free(checksum);
 
@@ -1774,7 +1749,6 @@ write_header_block_to_stream(off_t length_of_incoming_file,
 {
 	/*@ buffers **************************************************** */
 	char tempblock[TAPE_BLOCK_SIZE];
-	char *tmp = NULL;
 	char *p;
 
 	/*@ int ******************************************************** */
@@ -1784,6 +1758,9 @@ write_header_block_to_stream(off_t length_of_incoming_file,
 
 	/*@ end vars *************************************************** */
 
+	if (length_of_incoming_file <= 0) {
+		return(1);
+	}
 
 	olen = length_of_incoming_file;
 	p = strrchr(filename, '/');	/* Make 'em go, "Unnnh!" Oh wait, that was _Master_ P... */
@@ -1793,8 +1770,7 @@ write_header_block_to_stream(off_t length_of_incoming_file,
 		p++;
 	}
 	if (!g_tape_stream) {
-		log_to_screen
-			("You're not backing up to tape. Why write a tape header?");
+		log_to_screen("You're not backing up to tape. Why write a tape header?");
 		return (1);
 	}
 	for (i = 0; i < (int) TAPE_BLOCK_SIZE; i++) {
@@ -1804,14 +1780,8 @@ write_header_block_to_stream(off_t length_of_incoming_file,
 	tempblock[7000] = control_char;
 	memcpy(tempblock + 7001, (char *) &olen, sizeof(off_t));
 	strcpy(tempblock + 1000, filename);
-	g_tape_posK +=
-		fwrite(tempblock, 1, (size_t) TAPE_BLOCK_SIZE,
-			   g_tape_stream) / 1024;
-	mr_asprintf(&tmp, "%s (fname=%s, size=%ld K)",
-			marker_to_string(control_char), p,
-			(long) length_of_incoming_file >> 10);
-	log_msg(6, tmp);
-	mr_free(tmp);
+	g_tape_posK += fwrite(tempblock, 1, (size_t) TAPE_BLOCK_SIZE, g_tape_stream) / 1024;
+	log_msg(6, "%s (fname=%s, size=%ld K)", marker_to_string(control_char), p, (long) length_of_incoming_file >> 10);
 	return (0);
 }
 
@@ -1831,14 +1801,7 @@ write_header_block_to_stream(off_t length_of_incoming_file,
  */
 void wrong_marker(int should_be, int it_is)
 {
-	/*@ buffer ***************************************************** */
-	char *tmp = NULL;
-
-
-	/*@ end vars *************************************************** */
-	mr_asprintf(&tmp, "Wrong marker! (Should be %s, is actually %s)", marker_to_string(should_be), marker_to_string(it_is));
-	log_to_screen(tmp);
-	mr_free(tmp);
+	log_to_screen("Wrong marker! (Should be %s, is actually %s)", marker_to_string(should_be), marker_to_string(it_is));
 }
 
 /* @} - end of streamGroup */
