@@ -1,5 +1,5 @@
 /***************************************************************************
-    cvsid                : $Id: mondo-rstr-compare.c 3060 2012-11-10 04:05:37Z bruno $
+    cvsid                : $Id: mondo-rstr-compare.c 3273 2014-04-29 19:27:05Z bruno $
 ***************************************************************************/
 
 #include <pthread.h>
@@ -10,7 +10,7 @@
 //#include "../../config.h"
 #include "mr-externs.h"
 #include "mondo-rstr-compare.h"
-#include "mondo-restore-EXT.h"
+#include "mondorestore-EXT.h"
 #include "mondo-rstr-tools-EXT.h"
 
 extern char *MONDO_LOGFILE;
@@ -20,7 +20,7 @@ extern struct s_bkpinfo *bkpinfo;
 
 extern char *g_mountlist_fname;
 
-//static char cvsid[] = "$Id: mondo-rstr-compare.c 3060 2012-11-10 04:05:37Z bruno $";
+//static char cvsid[] = "$Id: mondo-rstr-compare.c 3273 2014-04-29 19:27:05Z bruno $";
 
 void popup_changelist_from_file(char *);
 
@@ -42,13 +42,11 @@ int compare_a_biggiefile(long bigfileno)
 	FILE *fout;
 
   /** needs malloc *******/
-	char *checksum_ptr;
-	char *original_cksum_ptr;
-	char *bigfile_fname_ptr;
-	char *tmp_ptr = NULL;
-	char *command_ptr;
-
-	char *checksum, *original_cksum, *bigfile_fname, *tmp, *command;
+	char *checksum = NULL;
+	char *original_cksum = NULL;
+	char *bigfile_fname = NULL;
+	char *tmp = NULL;
+	char *command = NULL;
 
 	char *p;
 	int i;
@@ -56,40 +54,22 @@ int compare_a_biggiefile(long bigfileno)
 
 	struct s_filename_and_lstat_info biggiestruct;
 
-	malloc_string(checksum);
-	malloc_string(original_cksum);
-	malloc_string(bigfile_fname);
-	malloc_string(tmp);
-	malloc_string(command);
-	malloc_string(checksum_ptr);
-	malloc_string(original_cksum_ptr);
-	malloc_string(bigfile_fname_ptr);
-	malloc_string(command_ptr);
-
   /*********************************************************************
    * allocate memory clear test                sab 16 feb 2003         *
    *********************************************************************/
 	assert(bkpinfo != NULL);
-	memset(checksum_ptr, '\0', sizeof(checksum));
-	memset(original_cksum_ptr, '\0', sizeof(original_cksum));
-	memset(bigfile_fname_ptr, '\0', sizeof(bigfile_fname));
-	memset(command_ptr, '\0', sizeof(command));
   /** end **/
 
 	if (!does_file_exist(slice_fname(bigfileno, 0, ARCHIVES_PATH, ""))) {
 		if (does_file_exist(MNT_CDROM "/archives/NOT-THE-LAST")) {
 			insist_on_this_cd_number((++g_current_media_number));
 		} else {
-			mr_asprintf(&tmp_ptr, "No CD's left. No biggiefiles left. No prob, Bob.");
-			log_msg(2, tmp_ptr);
-			paranoid_free(tmp_ptr);
+			log_msg(2, "No CD's left. No biggiefiles left. No prob, Bob.");
 			return (0);
 		}
 	}
 	if (!(fin = fopen(slice_fname(bigfileno, 0, ARCHIVES_PATH, ""), "r"))) {
-		mr_asprintf(&tmp_ptr, "Cannot open bigfile %ld (%s)'s info file", bigfileno + 1, bigfile_fname_ptr);
-		log_to_screen(tmp_ptr);
-		paranoid_free(tmp_ptr);
+		log_to_screen("Cannot open bigfile %ld (NULL)'s info file", bigfileno + 1);
 		return (1);
 	}
 	if (fread((void *) &biggiestruct, 1, sizeof(biggiestruct), fin)) {
@@ -97,84 +77,81 @@ int compare_a_biggiefile(long bigfileno)
 	}
 	paranoid_fclose(fin);
 
-	strcpy(checksum_ptr, biggiestruct.checksum);
-	strcpy(bigfile_fname_ptr, biggiestruct.filename);
 
+	mr_asprintf(bigfile_fname, "%s", biggiestruct.filename);
 	log_msg(2, "biggiestruct.filename = %s", biggiestruct.filename);
+	mr_asprintf(checksum, "%s", biggiestruct.checksum);
 	log_msg(2, "biggiestruct.checksum = %s", biggiestruct.checksum);
 
 	if (!g_text_mode) {
-		mr_asprintf(&tmp_ptr, "Comparing %s", bigfile_fname_ptr);
-		newtDrawRootText(0, 22, tmp_ptr);
-		paranoid_free(tmp_ptr);
+		mr_asprintf(tmp, "Comparing %s", bigfile_fname);
+		newtDrawRootText(0, 22, tmp);
+		mr_free(tmp);
 		newtRefresh();
 	}
-	if (!checksum[0]) {
-		log_msg(2, "Warning - %s has no checksum", bigfile_fname_ptr);
+	if (checksum == NULL) {
+		log_msg(2, "Warning - %s has no checksum", bigfile_fname);
 	}
-	if (!strncmp(bigfile_fname_ptr, "/dev/", 5)) {
-		strcpy(original_cksum_ptr, "IGNORE");
-	} else {
-		sprintf(command_ptr,
-				"md5sum \"%s%s\" > /tmp/md5sum.txt 2> /tmp/errors",
-				MNT_RESTORING, bigfile_fname_ptr);
+	if (!strncmp(bigfile_fname, "/dev/", 5)) {
+		log_msg(2, "IGNORING %s as begining with /dev", bigfile_fname);
+		mr_free(checksum);
+		mr_free(bigfile_fname);
+		return (1);
 	}
-	log_msg(2, command_ptr);
-	mr_asprintf(&tmp_ptr, "cat /tmp/errors >> %s 2> /dev/null", MONDO_LOGFILE);
-	paranoid_system(tmp_ptr);
-	paranoid_free(tmp_ptr);
 
-	if (system(command_ptr)) {
+	mr_asprintf(command, "md5sum \"%s%s\" > /tmp/md5sum.txt 2> /tmp/errors", MNT_RESTORING, bigfile_fname);
+	log_msg(2, command);
+	i =  system(command);
+	mr_free(command);
+
+	mr_asprintf(tmp, "cat /tmp/errors >> %s 2> /dev/null", MONDO_LOGFILE);
+	paranoid_system(tmp);
+	mr_free(tmp);
+
+	if (i) {
 		log_OS_error("Warning - command failed");
-		original_cksum[0] = '\0';
+		mr_free(checksum);
+		mr_free(bigfile_fname);
 		return (1);
 	} else {
 		if (!(fin = fopen("/tmp/md5sum.txt", "r"))) {
-			log_msg(2,
-					"Unable to open /tmp/md5sum.txt; can't get live checksum");
-			original_cksum[0] = '\0';
+			log_msg(2, "Unable to open /tmp/md5sum.txt; can't get live checksum");
+			mr_free(checksum);
+			mr_free(bigfile_fname);
 			return (1);
 		} else {
-			if (fgets(original_cksum_ptr, MAX_STR_LEN - 1, fin)) {
-				// FIXME
-			}
+			mr_getline(original_cksum, fin);
 			paranoid_fclose(fin);
-			for (i = strlen(original_cksum_ptr);
+			for (i = strlen(original_cksum);
 				 i > 0 && original_cksum[i - 1] < 32; i--);
 			original_cksum[i] = '\0';
-			p = (char *) strchr(original_cksum_ptr, ' ');
+			p = strchr(original_cksum, ' ');
 			if (p) {
 				*p = '\0';
 			}
 		}
 	}
-	mr_asprintf(&tmp_ptr, "bigfile #%ld ('%s') ", bigfileno + 1, bigfile_fname_ptr);
-	if (!strcmp(checksum_ptr, original_cksum_ptr) != 0) {
-		mr_strcat(tmp_ptr, " ... OK");
+	mr_asprintf(tmp, "bigfile #%ld ('%s') ", bigfileno + 1, bigfile_fname);
+	if ((original_cksum != NULL) && (strcmp(checksum, original_cksum) == 0)) {
+		mr_strcat(tmp, " ... OK");
 	} else {
-		mr_strcat(tmp_ptr, "... changed");
+		mr_strcat(tmp, "... changed");
 		retval++;
 	}
-	log_msg(1, tmp_ptr);
-	paranoid_free(tmp_ptr);
+	mr_free(checksum);
+	mr_free(original_cksum);
+
+	log_msg(1, tmp);
+	mr_free(tmp);
 
 	if (retval) {
 		if (!(fout = fopen(MONDO_CACHE"/changed.txt", "a"))) {
 			fatal_error("Cannot openout changed.txt");
 		}
-		fprintf(fout, "%s\n", bigfile_fname_ptr);
+		fprintf(fout, "%s\n", bigfile_fname);
 		paranoid_fclose(fout);
 	}
-
-	paranoid_free(original_cksum_ptr);
-	paranoid_free(original_cksum);
-	paranoid_free(bigfile_fname_ptr);
-	paranoid_free(bigfile_fname);
-	paranoid_free(checksum_ptr);
-	paranoid_free(checksum);
-	paranoid_free(command_ptr);
-	paranoid_free(command);
-	paranoid_free(tmp);
+	mr_free(bigfile_fname);
 
 	return (retval);
 }
@@ -193,7 +170,7 @@ int compare_all_biggiefiles()
 	int retval = 0;
 	int res;
 	long noof_biggiefiles, bigfileno = 0;
-	char tmp[MAX_STR_LEN];
+	char *tmp = NULL;
 
 	log_msg(1, "Comparing biggiefiles");
 
@@ -214,9 +191,11 @@ int compare_all_biggiefiles()
 					   "against the filesystem. Please wait.", "",
 					   noof_biggiefiles);
 	for (bigfileno = 0; bigfileno < noof_biggiefiles; bigfileno++) {
-		sprintf(tmp, "Comparing big file #%ld", bigfileno + 1);
+		mr_asprintf(tmp, "Comparing big file #%ld", bigfileno + 1);
 		log_msg(1, tmp);
 		update_progress_form(tmp);
+		mr_free(tmp);
+
 		res = compare_a_biggiefile(bigfileno);
 		retval += res;
 		g_current_progress++;
@@ -252,76 +231,73 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 	bool use_star;
 
   /***  needs malloc *********/
-	char *command, *tmp, *filelist_name, *logfile,
-		*compressor_exe;
+	char *command = NULL;
+	char *tmp = NULL;
 	char *archiver_exe = NULL;
-
-	malloc_string(command);
-	malloc_string(tmp);
-	malloc_string(filelist_name);
-	malloc_string(logfile);
-	malloc_string(compressor_exe);
+	char *logfile = NULL;
+	char *compressor_exe = NULL;
 
 	use_star = (strstr(tarball_fname, ".star")) ? TRUE : FALSE;
 	assert_string_is_neither_NULL_nor_zerolength(tarball_fname);
-	sprintf(logfile, "/tmp/afio.log.%d", current_tarball_number);
-	sprintf(filelist_name, MNT_CDROM "/archives/filelist.%d",
-			current_tarball_number);
 
 	if (strstr(tarball_fname, ".bz2")) {
-		strcpy(compressor_exe, "bzip2");
+		mr_asprintf(compressor_exe, "bzip2");
+	} else if (strstr(tarball_fname, ".lzma")) {
+		mr_asprintf(compressor_exe, "lzma");
 	} else if (strstr(tarball_fname, ".gz")) {
-		strcpy(compressor_exe, "gzip");
+		mr_asprintf(compressor_exe, "gzip");
 	} else if (strstr(tarball_fname, ".lzo")) {
-		strcpy(compressor_exe, "lzop");
-	} else {
-		compressor_exe[0] = '\0';
+		mr_asprintf(compressor_exe, "lzop");
 	}
 
 	if (use_star) {
-		mr_asprintf(&archiver_exe, "star");
+		mr_asprintf(archiver_exe, "star");
 	} else {
-		mr_asprintf(&archiver_exe, "afio");
+		mr_asprintf(archiver_exe, "afio");
 	}
 
-	if (compressor_exe[0]) {
-		strcpy(tmp, compressor_exe);
+	if (compressor_exe) {
+		mr_asprintf(tmp, "%s", compressor_exe);
 		if (!find_home_of_exe(tmp)) {
+			mr_free(tmp);
+			mr_free(compressor_exe);
+			mr_free(archiver_exe);
 			fatal_error("(compare_a_tarball) Compression program missing");
 		}
-		if (use_star)			// star
-		{
+		mr_free(tmp);
+
+		if (use_star) {
 			if (!strcmp(compressor_exe, "bzip2")) {
 				mr_strcat(archiver_exe, " -bz");
 			} else {
-				fatal_error
-					("(compare_a_tarball) Please use only bzip2 with star");
+				mr_free(compressor_exe);
+				mr_free(archiver_exe);
+				fatal_error("(compare_a_tarball) Please use only bzip2 with star");
 			}
-		} else					// afio
-		{
-			sprintf(compressor_exe, "-P %s -Z", tmp);
+		} else {
+			// afio
+			mr_asprintf(tmp, "%s", compressor_exe);
+			mr_free(compressor_exe);
+			mr_asprintf(compressor_exe, "-P %s -Z", tmp);
+			mr_free(tmp);
 		}
 	}
-// star -diff H=star -bz file=....
 
 #ifdef __FreeBSD__
 #define BUFSIZE 512L
 #else
 #define BUFSIZE (1024L*1024L)/TAPE_BLOCK_SIZE
 #endif
-	if (use_star)				// doesn't use compressor_exe
-	{
-		sprintf(command,
-				"%s -sparse -diff H=exustar file=%s >> %s 2>> %s",
-				archiver_exe, tarball_fname, logfile, logfile);
+	mr_asprintf(logfile, "/tmp/afio.log.%d", current_tarball_number);
+
+	if (use_star) {
+		// doesn't use compressor_exe
+		mr_asprintf(command, "%s -sparse -diff H=exustar file=%s >> %s 2>> %s", archiver_exe, tarball_fname, logfile, logfile);
 	} else {
-		sprintf(command,
-				"%s -r -b %ld -M 16m -c %ld %s %s >> %s 2>> %s",
-				archiver_exe,
-				TAPE_BLOCK_SIZE,
-				BUFSIZE, compressor_exe, tarball_fname, logfile, logfile);
+		mr_asprintf(command, "%s -r -b %ld -M 16m -c %ld %s %s >> %s 2>> %s", archiver_exe, TAPE_BLOCK_SIZE, BUFSIZE, compressor_exe, tarball_fname, logfile, logfile);
 	}
-	paranoid_free(archiver_exe);
+	mr_free(compressor_exe);
+	mr_free(archiver_exe);
 
 #undef BUFSIZE
 
@@ -329,33 +305,25 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 	retval += res;
 	if (res) {
 		log_OS_error(command);
-		sprintf(tmp, "Warning - afio returned error = %d", res);
-		log_msg(2, tmp);
+		log_msg(2, "Warning - afio returned error = %d", res);
 	}
+	mr_free(command);
+
 	if (length_of_file(logfile) > 5) {
-		sprintf(command,
-				"sed s/': \\\"'/\\|/ %s | sed s/'\\\": '/\\|/ | cut -d'|' -f2 | sort -u | grep -vE \"^dev/.*\" >> "MONDO_CACHE"/changed.txt",
-				logfile);
+		mr_asprintf(command, "sed s/': \\\"'/\\|/ %s | sed s/'\\\": '/\\|/ | cut -d'|' -f2 | sort -u | grep -vE \"^dev/.*\" >> "MONDO_CACHE"/changed.txt", logfile);
 		paranoid_system(command);
+		mr_free(command);
+
 		archiver_errors = count_lines_in_file(logfile);
 	} else {
 		archiver_errors = 0;
 	}
-	sprintf(tmp, "%ld difference%c in fileset #%d          ",
-			archiver_errors, (archiver_errors != 1) ? 's' : ' ',
-			current_tarball_number);
 	if (archiver_errors) {
-		sprintf(tmp,
-				"Differences found while processing fileset #%d       ",
-				current_tarball_number);
-		log_msg(1, tmp);
+		log_msg(1, "%ld difference%c in fileset #%d          ", archiver_errors, (archiver_errors != 1) ? 's' : ' ', current_tarball_number);
 	}
 	unlink(logfile);
-	paranoid_free(command);
-	paranoid_free(tmp);
-	paranoid_free(filelist_name);
-	paranoid_free(logfile);
-	paranoid_free(compressor_exe);
+	mr_free(logfile);
+
 	return (retval);
 }
 
@@ -376,21 +344,21 @@ int compare_all_tarballs()
 
   /**  needs malloc **********/
 
-	char *tarball_fname, *progress_str, *tmp;
+	char *tarball_fname = NULL;
+	char *progress_str = NULL; 
+	char *tmp = NULL;
 	char *mds = NULL;
 	long max_val;
 
-	malloc_string(tarball_fname);
-	malloc_string(progress_str);
 	malloc_string(tmp);
-
 	assert(bkpinfo != NULL);
 	mvaddstr_and_log_it(g_currentY, 0, "Comparing archives");
 	read_cfg_var(g_mondo_cfg_file, "last-filelist-number", tmp);
-
 	max_val = atol(tmp);
+	paranoid_free(tmp);
+
 	mds = media_descriptor_string(bkpinfo->backup_media_type);
-	sprintf(progress_str, "Comparing with %s #%d ", mds, g_current_media_number);
+	mr_asprintf(progress_str, "Comparing with %s #%d ", mds, g_current_media_number);
 
 	open_progress_form("Comparing files",
 					   "Comparing tarballs against filesystem.",
@@ -402,28 +370,31 @@ int compare_all_tarballs()
 	for (;;) {
 		insist_on_this_cd_number(g_current_media_number);
 		update_progress_form(progress_str);
-		sprintf(tarball_fname,
-				MNT_CDROM "/archives/%d.afio.bz2", current_tarball_number);
+		mr_asprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.bz2", current_tarball_number);
 
 		if (!does_file_exist(tarball_fname)) {
-			sprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.lzo",
-					current_tarball_number);
+			mr_free(tarball_fname);
+			mr_asprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.lzo", current_tarball_number);
 		}
 		if (!does_file_exist(tarball_fname)) {
-			sprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.gz",
-					current_tarball_number);
+			mr_free(tarball_fname);
+			mr_asprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.lzma", current_tarball_number);
 		}
 		if (!does_file_exist(tarball_fname)) {
-			sprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.",
-					current_tarball_number);
+			mr_free(tarball_fname);
+			mr_asprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.gz", current_tarball_number);
 		}
 		if (!does_file_exist(tarball_fname)) {
-			sprintf(tarball_fname, MNT_CDROM "/archives/%d.star.bz2",
-					current_tarball_number);
+			mr_free(tarball_fname);
+			mr_asprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.", current_tarball_number);
 		}
 		if (!does_file_exist(tarball_fname)) {
-			sprintf(tarball_fname, MNT_CDROM "/archives/%d.star.",
-					current_tarball_number);
+			mr_free(tarball_fname);
+			mr_asprintf(tarball_fname, MNT_CDROM "/archives/%d.star.bz2", current_tarball_number);
+		}
+		if (!does_file_exist(tarball_fname)) {
+			mr_free(tarball_fname);
+			mr_asprintf(tarball_fname, MNT_CDROM "/archives/%d.star.", current_tarball_number);
 		}
 		if (!does_file_exist(tarball_fname)) {
 			if (!does_file_exist(MNT_CDROM "/archives/NOT-THE-LAST") ||
@@ -431,11 +402,14 @@ int compare_all_tarballs()
 					   "/archives/slice* > /dev/null 2> /dev/null")
 				== 0) {
 				log_msg(2, "OK, I think I'm done with tarballs...");
+				mr_free(tarball_fname);
 				break;
 			}
 			log_msg(2, "OK, I think it's time for another CD...");
 			g_current_media_number++;
-			sprintf(progress_str, "Comparing with %s #%d ", mds, g_current_media_number);
+
+			mr_free(progress_str);
+			mr_asprintf(progress_str, "Comparing with %s #%d ", mds, g_current_media_number);
 			log_to_screen(progress_str);
 		} else {
 			retval += compare_a_tarball(tarball_fname, current_tarball_number);
@@ -443,7 +417,9 @@ int compare_all_tarballs()
 			g_current_progress++;
 			current_tarball_number++;
 		}
+		mr_free(tarball_fname);
 	}
+	mr_free(progress_str);
 	mr_free(mds);
 
 	close_progress_form();
@@ -452,9 +428,6 @@ int compare_all_tarballs()
 	} else {
 		mvaddstr_and_log_it(g_currentY++, 74, "Done.");
 	}
-	paranoid_free(tarball_fname);
-	paranoid_free(progress_str);
-	paranoid_free(tmp);
 	return (retval);
 }
 
@@ -477,7 +450,9 @@ int compare_all_tarballs()
 int compare_to_CD()
 {
   /** needs malloc *********/
-	char *tmp, *cwd, *new, *command;
+	char *tmp = NULL;
+	char *cwd, *new;
+	char *command = NULL;
 	int resA = 0;
 	int resB = 0;
 	long noof_changed_files;
@@ -485,7 +460,6 @@ int compare_to_CD()
 	malloc_string(tmp);
 	malloc_string(cwd);
 	malloc_string(new);
-	malloc_string(command);
 
 	assert(bkpinfo != NULL);
 
@@ -508,22 +482,20 @@ int compare_to_CD()
 	}
 	noof_changed_files = count_lines_in_file(MONDO_CACHE"/changed.txt");
 	if (noof_changed_files) {
-		sprintf(tmp, "%ld files do not match the backup            ",
-				noof_changed_files);
-		//      mvaddstr_and_log_it( g_currentY++, 0, tmp );
-		log_to_screen(tmp);
-		sprintf(command, "cat "MONDO_CACHE"/changed.txt >> %s", MONDO_LOGFILE);
+		log_to_screen("%ld files do not match the backup            ", noof_changed_files);
+
+		mr_asprintf(command, "cat "MONDO_CACHE"/changed.txt >> %s", MONDO_LOGFILE);
 		paranoid_system(command);
+		mr_free(command);
 	} else {
-		sprintf(tmp, "All files match the backup                     ");
+		mr_asprintf(tmp, "All files match the backup                     ");
 		mvaddstr_and_log_it(g_currentY++, 0, tmp);
 		log_to_screen(tmp);
+		mr_free(tmp);
 	}
 
-	paranoid_free(tmp);
 	paranoid_free(cwd);
 	paranoid_free(new);
-	paranoid_free(command);
 
 	return (resA + resB);
 }
@@ -551,11 +523,10 @@ compare_mode(struct mountlist_itself *mountlist,
 	int retval = 0;
 	int res = 0;
 	long q;
-	char *tmp;
+	char *tmp = NULL;
 	char *new;
 	char *cwd;
 
-	malloc_string(tmp);
 	malloc_string(new);
 	malloc_string(cwd);
 
@@ -629,15 +600,15 @@ compare_mode(struct mountlist_itself *mountlist,
 				("The differences were logfiles and temporary files. Your archives are fine.");
 		} else {
 			q = count_lines_in_file(MONDO_CACHE"/changed.files");
-			sprintf(tmp, "%ld significant difference%s found.", q,
-					(q != 1) ? "s" : "");
+			mr_asprintf(tmp, "%ld significant difference%s found.", q, (q != 1) ? "s" : "");
 			mvaddstr_and_log_it(g_currentY++, 0, tmp);
 			log_to_screen(tmp);
+			mr_free(tmp);
 
-			strcpy(tmp,
-				   "Type 'less /tmp/changed.files' for a list of non-matching files");
+			mr_asprintf(tmp, "Type 'less /tmp/changed.files' for a list of non-matching files");
 			mvaddstr_and_log_it(g_currentY++, 0, tmp);
 			log_to_screen(tmp);
+			mr_free(tmp);
 
 			log_msg(2, "calling popup_changelist_from_file()");
 			if (getcwd(cwd, MAX_STR_LEN - 1)) {
@@ -662,7 +633,6 @@ compare_mode(struct mountlist_itself *mountlist,
 	retval += unmount_all_devices(mountlist);
 
 	kill_petris();
-	paranoid_free(tmp);
 	paranoid_free(new);
 	paranoid_free(cwd);
 	return (retval);
@@ -685,11 +655,11 @@ int compare_to_cdstream()
 	int res;
 
   /** needs malloc **/
-	char *dir, *command;
+	char *dir;
+	char *command = NULL;
 
 	assert(bkpinfo != NULL);
 	malloc_string(dir);
-	malloc_string(command);
 	if (getcwd(dir, MAX_STR_LEN)) {
 		// FIXME
 	}
@@ -697,11 +667,10 @@ int compare_to_cdstream()
 		// FIXME
 	}
 
-	sprintf(command, "cp -f /tmp/LAST-FILELIST-NUMBER %s/tmp",
-			bkpinfo->restore_path);
+	mr_asprintf(command, "cp -f /tmp/LAST-FILELIST-NUMBER %s/tmp", bkpinfo->restore_path);
 	run_program_and_log_output(command, FALSE);
-	mvaddstr_and_log_it(g_currentY,
-						0, "Verifying archives against filesystem");
+	mr_free(command);
+	mvaddstr_and_log_it(g_currentY,	0, "Verifying archives against filesystem");
 
 	if (bkpinfo->disaster_recovery
 		&& does_file_exist("/tmp/CDROM-LIVES-HERE")) {
@@ -725,7 +694,6 @@ int compare_to_cdstream()
 
 	mvaddstr_and_log_it(g_currentY++, 74, "Done.");
 	paranoid_free(dir);
-	paranoid_free(command);
 	return (res);
 }
 
@@ -748,11 +716,11 @@ int compare_to_cdstream()
 int compare_to_tape()
 {
 	int res;
-	char *dir, *command;
+	char *dir;
+	char *command = NULL;
 
 	assert(bkpinfo != NULL);
 	malloc_string(dir);
-	malloc_string(command);
 
 	if (getcwd(dir, MAX_STR_LEN)) {
 		// FIXME
@@ -760,11 +728,11 @@ int compare_to_tape()
 	if (chdir(bkpinfo->restore_path)) {
 		// FIXME
 	}
-	sprintf(command, "cp -f /tmp/LAST-FILELIST-NUMBER %s/tmp",
-			bkpinfo->restore_path);
+	mr_asprintf(command, "cp -f /tmp/LAST-FILELIST-NUMBER %s/tmp", bkpinfo->restore_path);
 	run_program_and_log_output(command, FALSE);
-	mvaddstr_and_log_it(g_currentY,
-						0, "Verifying archives against filesystem");
+	mr_free(command);
+
+	mvaddstr_and_log_it(g_currentY,	0, "Verifying archives against filesystem");
 	res = verify_tape_backups();
 	if (chdir(dir)) {
 		// FIXME
@@ -775,7 +743,6 @@ int compare_to_tape()
 		mvaddstr_and_log_it(g_currentY++, 74, "Done.");
 	}
 	paranoid_free(dir);
-	paranoid_free(command);
 	return (res);
 }
 
